/* structure.c generated by valac 0.15.1, the Vala compiler
 * generated from structure.vala, do not modify */

/*
 * This file is part of LaTeXila.
 *
 * Copyright © 2011 Sébastien Wilmet
 *
 * LaTeXila is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * LaTeXila is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with LaTeXila.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <gtksourceview/gtksourceview.h>
#include <gio/gio.h>
#include <gdk/gdk.h>
#include <float.h>
#include <math.h>


#define TYPE_STRUCT_TYPE (struct_type_get_type ())

#define TYPE_STRUCT_ACTION (struct_action_get_type ())

#define TYPE_STRUCTURE (structure_get_type ())
#define STRUCTURE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_STRUCTURE, Structure))
#define STRUCTURE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_STRUCTURE, StructureClass))
#define IS_STRUCTURE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_STRUCTURE))
#define IS_STRUCTURE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_STRUCTURE))
#define STRUCTURE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_STRUCTURE, StructureClass))

typedef struct _Structure Structure;
typedef struct _StructureClass StructureClass;
typedef struct _StructurePrivate StructurePrivate;

#define TYPE_MAIN_WINDOW (main_window_get_type ())
#define MAIN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MAIN_WINDOW, MainWindow))
#define MAIN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MAIN_WINDOW, MainWindowClass))
#define IS_MAIN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MAIN_WINDOW))
#define IS_MAIN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MAIN_WINDOW))
#define MAIN_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MAIN_WINDOW, MainWindowClass))

typedef struct _MainWindow MainWindow;
typedef struct _MainWindowClass MainWindowClass;

#define TYPE_DOCUMENT_STRUCTURE (document_structure_get_type ())
#define DOCUMENT_STRUCTURE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DOCUMENT_STRUCTURE, DocumentStructure))
#define DOCUMENT_STRUCTURE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DOCUMENT_STRUCTURE, DocumentStructureClass))
#define IS_DOCUMENT_STRUCTURE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DOCUMENT_STRUCTURE))
#define IS_DOCUMENT_STRUCTURE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DOCUMENT_STRUCTURE))
#define DOCUMENT_STRUCTURE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DOCUMENT_STRUCTURE, DocumentStructureClass))

typedef struct _DocumentStructure DocumentStructure;
typedef struct _DocumentStructureClass DocumentStructureClass;

#define TYPE_STRUCTURE_MODEL (structure_model_get_type ())
#define STRUCTURE_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_STRUCTURE_MODEL, StructureModel))
#define STRUCTURE_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_STRUCTURE_MODEL, StructureModelClass))
#define IS_STRUCTURE_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_STRUCTURE_MODEL))
#define IS_STRUCTURE_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_STRUCTURE_MODEL))
#define STRUCTURE_MODEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_STRUCTURE_MODEL, StructureModelClass))

typedef struct _StructureModel StructureModel;
typedef struct _StructureModelClass StructureModelClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define TYPE_DOCUMENT (document_get_type ())
#define DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DOCUMENT, Document))
#define DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DOCUMENT, DocumentClass))
#define IS_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DOCUMENT))
#define IS_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DOCUMENT))
#define DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DOCUMENT, DocumentClass))

typedef struct _Document Document;
typedef struct _DocumentClass DocumentClass;
typedef struct _Block21Data Block21Data;
#define _g_free0(var) (var = (g_free (var), NULL))
#define __g_list_free__gtk_tree_path_free0_0(var) ((var == NULL) ? NULL : (var = (_g_list_free__gtk_tree_path_free0_ (var), NULL)))
#define _gtk_tree_path_free0(var) ((var == NULL) ? NULL : (var = (gtk_tree_path_free (var), NULL)))

#define TYPE_STRUCT_LIST_COLUMN (struct_list_column_get_type ())

#define TYPE_STRUCT_COLUMN (struct_column_get_type ())

#define TYPE_DOCUMENT_VIEW (document_view_get_type ())
#define DOCUMENT_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DOCUMENT_VIEW, DocumentView))
#define DOCUMENT_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DOCUMENT_VIEW, DocumentViewClass))
#define IS_DOCUMENT_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DOCUMENT_VIEW))
#define IS_DOCUMENT_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DOCUMENT_VIEW))
#define DOCUMENT_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DOCUMENT_VIEW, DocumentViewClass))

typedef struct _DocumentView DocumentView;
typedef struct _DocumentViewClass DocumentViewClass;
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

typedef enum  {
	STRUCT_TYPE_PART,
	STRUCT_TYPE_CHAPTER,
	STRUCT_TYPE_SECTION,
	STRUCT_TYPE_SUBSECTION,
	STRUCT_TYPE_SUBSUBSECTION,
	STRUCT_TYPE_PARAGRAPH,
	STRUCT_TYPE_SUBPARAGRAPH,
	STRUCT_TYPE_LABEL,
	STRUCT_TYPE_INCLUDE,
	STRUCT_TYPE_IMAGE,
	STRUCT_TYPE_TODO,
	STRUCT_TYPE_FIXME,
	STRUCT_TYPE_NB_COMMON_TYPES,
	STRUCT_TYPE_TABLE,
	STRUCT_TYPE_FIGURE,
	STRUCT_TYPE_NB_TYPES
} StructType;

typedef enum  {
	STRUCT_ACTION_CUT,
	STRUCT_ACTION_COPY,
	STRUCT_ACTION_DELETE,
	STRUCT_ACTION_SELECT,
	STRUCT_ACTION_COMMENT,
	STRUCT_ACTION_SHIFT_LEFT,
	STRUCT_ACTION_SHIFT_RIGHT,
	STRUCT_ACTION_NB_ACTIONS
} StructAction;

struct _Structure {
	GtkVBox parent_instance;
	StructurePrivate * priv;
};

struct _StructureClass {
	GtkVBoxClass parent_class;
};

struct _StructurePrivate {
	MainWindow* _main_window;
	GtkMenu* _popup_menu;
	GtkAction* _action_all_menu;
	GtkAction* _action_cut;
	GtkAction* _action_copy;
	GtkAction* _action_delete;
	GtkAction* _action_select;
	GtkAction* _action_comment;
	GtkAction* _action_shift_left;
	GtkAction* _action_shift_right;
	GtkToggleButton** _simple_list_buttons;
	gint _simple_list_buttons_length1;
	gint __simple_list_buttons_size_;
	GtkVPaned* _vpaned;
	GtkTreeView* _tree_view;
	DocumentStructure* _document_structure;
	StructureModel* _model;
	GtkTreeView* _list_view;
	GtkWidget* _list_view_sw;
	GtkListStore* _list_store;
	StructType* _current_list_types;
	gint _current_list_types_length1;
	gint __current_list_types_size_;
	gboolean _list_is_hidden;
	gboolean _first_select;
};

struct _Block21Data {
	int _ref_count_;
	Structure * self;
	StructType main_type;
	GtkToggleButton* button;
	StructType* types;
	gint types_length1;
};

typedef enum  {
	STRUCT_LIST_COLUMN_PIXBUF,
	STRUCT_LIST_COLUMN_TEXT,
	STRUCT_LIST_COLUMN_TOOLTIP,
	STRUCT_LIST_COLUMN_N_COLUMNS
} StructListColumn;

typedef enum  {
	STRUCT_COLUMN_PIXBUF,
	STRUCT_COLUMN_TEXT,
	STRUCT_COLUMN_TOOLTIP,
	STRUCT_COLUMN_START_MARK,
	STRUCT_COLUMN_END_MARK,
	STRUCT_COLUMN_TYPE,
	STRUCT_COLUMN_N_COLUMNS
} StructColumn;

typedef enum  {
	STRUCT_ERROR_GENERAL,
	STRUCT_ERROR_DATA_OUTDATED
} StructError;
#define STRUCT_ERROR struct_error_quark ()

static gpointer structure_parent_class = NULL;
static gchar** structure__icons;
static gint structure__icons_length1;
static gchar** structure__icons = NULL;
static gint structure__icons_length1 = 0;
static gint _structure__icons_size_ = 0;
static gchar** structure__names;
static gint structure__names_length1;
static gchar** structure__names = NULL;
static gint structure__names_length1 = 0;
static gint _structure__names_size_ = 0;
static gchar** structure__action_names;
static gint structure__action_names_length1;
static gchar** structure__action_names = NULL;
static gint structure__action_names_length1 = 0;
static gint _structure__action_names_size_ = 0;

GType struct_type_get_type (void) G_GNUC_CONST;
GType struct_action_get_type (void) G_GNUC_CONST;
GType structure_get_type (void) G_GNUC_CONST;
GType main_window_get_type (void) G_GNUC_CONST;
GType document_structure_get_type (void) G_GNUC_CONST;
GType structure_model_get_type (void) G_GNUC_CONST;
#define STRUCTURE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_STRUCTURE, StructurePrivate))
enum  {
	STRUCTURE_DUMMY_PROPERTY
};
Structure* structure_new (MainWindow* main_window, GtkUIManager* ui_manager);
Structure* structure_construct (GType object_type, MainWindow* main_window, GtkUIManager* ui_manager);
static void structure_init_toolbar (Structure* self);
static void structure_init_vpaned (Structure* self);
static void structure_init_list_view (Structure* self);
static void structure_init_tree_view (Structure* self);
void structure_connect_parsing (Structure* self);
static void _structure_connect_parsing_gtk_widget_show (GtkWidget* _sender, gpointer self);
static void __lambda130_ (Structure* self);
void structure_disconnect_parsing (Structure* self);
static void ___lambda130__gtk_widget_hide (GtkWidget* _sender, gpointer self);
GtkButton* utils_get_toolbar_button (const gchar* stock_id);
static void __lambda119_ (Structure* self);
GType document_get_type (void) G_GNUC_CONST;
static void structure_show_document (Structure* self, Document* doc, gboolean force_parse);
Document* main_window_get_active_document (MainWindow* self);
static void ___lambda119__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void __lambda121_ (Structure* self);
static void ___lambda121__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void __lambda122_ (Structure* self);
static void ___lambda122__gtk_button_clicked (GtkButton* _sender, gpointer self);
static GtkToggleButton* structure_create_simple_list_button (Structure* self, StructType* types, int types_length1, const gchar* tooltip);
static Block21Data* block21_data_ref (Block21Data* _data21_);
static void block21_data_unref (Block21Data* _data21_);
GtkToggleButton* utils_get_toolbar_toggle_button (const gchar* stock_id);
gchar* structure_get_icon_from_type (StructType type);
static void _vala_array_add15 (GtkToggleButton*** array, int* length, int* size, GtkToggleButton* value);
static void __lambda123_ (Block21Data* _data21_);
static gboolean _vala_struct_type_array_contains (StructType* stack, int stack_length, StructType needle);
static StructType* _vala_array_dup3 (StructType* self, int length);
static void structure_populate_simple_list (Structure* self);
static void ___lambda123__gtk_button_clicked (GtkButton* _sender, gpointer self);
void structure_model_populate_list (StructureModel* self, GtkListStore* store, StructType type);
static void _gtk_tree_path_free0_ (gpointer var);
static void _g_list_free__gtk_tree_path_free0_ (GList* self);
static void structure_select_simple_list_item (Structure* self, GtkTreeIter* tree_iter);
void structure_save_state (Structure* self);
static GtkTreeView* structure_get_new_tree_view (Structure* self, gint pixbuf_col, gint text_col, gint tooltip_col);
GType struct_list_column_get_type (void) G_GNUC_CONST;
static gboolean __lambda124_ (Structure* self, GtkTreeSelection* select, GtkTreeModel* model, GtkTreePath* path, gboolean path_currently_selected);
static gboolean structure_select_list_row (Structure* self, GtkTreePath* list_path);
static gboolean ___lambda124__gtk_tree_selection_func (GtkTreeSelection* selection, GtkTreeModel* model, GtkTreePath* path, gboolean path_currently_selected, gpointer self);
static void __lambda125_ (Structure* self, GtkTreePath* path);
static void ___lambda125__gtk_tree_view_row_activated (GtkTreeView* _sender, GtkTreePath* path, GtkTreeViewColumn* column, gpointer self);
GtkWidget* utils_add_scrollbar (GtkWidget* child);
GType struct_column_get_type (void) G_GNUC_CONST;
static gboolean __lambda126_ (Structure* self, GtkTreeSelection* select, GtkTreeModel* model, GtkTreePath* path, gboolean path_currently_selected);
static gboolean structure_select_tree_row (Structure* self, GtkTreePath* tree_path);
static gboolean ___lambda126__gtk_tree_selection_func (GtkTreeSelection* selection, GtkTreeModel* model, GtkTreePath* path, gboolean path_currently_selected, gpointer self);
static void __lambda127_ (Structure* self, GtkTreePath* path);
static void ___lambda127__gtk_tree_view_row_activated (GtkTreeView* _sender, GtkTreePath* path, GtkTreeViewColumn* column, gpointer self);
static gboolean __lambda128_ (Structure* self, GdkEventButton* event);
static void structure_show_popup_menu (Structure* self, GdkEventButton* event);
static gboolean ___lambda128__gtk_widget_button_press_event (GtkWidget* _sender, GdkEventButton* event, gpointer self);
static gboolean __lambda129_ (Structure* self);
static gboolean ___lambda129__gtk_widget_popup_menu (GtkWidget* _sender, gpointer self);
GtkTreePath* structure_model_get_tree_path_from_list_num (StructureModel* self, StructType list_type, gint num);
GType document_view_get_type (void) G_GNUC_CONST;
DocumentView* main_window_get_active_view (MainWindow* self);
static void structure_set_actions_sensitivity (Structure* self, StructType type);
gint structure_model_get_list_num_from_tree_iter (StructureModel* self, GtkTreeIter* tree_iter);
static void structure_show_active_document (Structure* self);
static void structure_set_model (Structure* self, StructureModel* model);
static void structure_on_parsing_done (Structure* self);
static void _structure_on_parsing_done_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
DocumentStructure* document_get_structure (Document* self);
void document_structure_parse (DocumentStructure* self);
gboolean document_structure_get_parsing_done (DocumentStructure* self);
StructureModel* document_structure_get_model (DocumentStructure* self);
void structure_model_set_emit_signals (StructureModel* self, gboolean value);
guint structure_model_get_nb_items (StructureModel* self);
static void _structure_show_active_document_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
gboolean structure_is_section (StructType type);
void structure_do_action (Structure* self, StructAction action_type);
gint utils_get_selected_row (GtkTreeView* view, GtkTreeIter* iter);
GQuark struct_error_quark (void);
void document_structure_do_action (DocumentStructure* self, StructAction action_type, GtkTreeIter* tree_iter, gboolean* refresh_simple_list, GError** error);
static gchar* structure_get_action_name (StructAction action_type);
gchar* structure_get_type_name (StructType type);
static void structure_finalize (GObject* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);


GType struct_type_get_type (void) {
	static volatile gsize struct_type_type_id__volatile = 0;
	if (g_once_init_enter (&struct_type_type_id__volatile)) {
		static const GEnumValue values[] = {{STRUCT_TYPE_PART, "STRUCT_TYPE_PART", "part"}, {STRUCT_TYPE_CHAPTER, "STRUCT_TYPE_CHAPTER", "chapter"}, {STRUCT_TYPE_SECTION, "STRUCT_TYPE_SECTION", "section"}, {STRUCT_TYPE_SUBSECTION, "STRUCT_TYPE_SUBSECTION", "subsection"}, {STRUCT_TYPE_SUBSUBSECTION, "STRUCT_TYPE_SUBSUBSECTION", "subsubsection"}, {STRUCT_TYPE_PARAGRAPH, "STRUCT_TYPE_PARAGRAPH", "paragraph"}, {STRUCT_TYPE_SUBPARAGRAPH, "STRUCT_TYPE_SUBPARAGRAPH", "subparagraph"}, {STRUCT_TYPE_LABEL, "STRUCT_TYPE_LABEL", "label"}, {STRUCT_TYPE_INCLUDE, "STRUCT_TYPE_INCLUDE", "include"}, {STRUCT_TYPE_IMAGE, "STRUCT_TYPE_IMAGE", "image"}, {STRUCT_TYPE_TODO, "STRUCT_TYPE_TODO", "todo"}, {STRUCT_TYPE_FIXME, "STRUCT_TYPE_FIXME", "fixme"}, {STRUCT_TYPE_NB_COMMON_TYPES, "STRUCT_TYPE_NB_COMMON_TYPES", "nb-common-types"}, {STRUCT_TYPE_TABLE, "STRUCT_TYPE_TABLE", "table"}, {STRUCT_TYPE_FIGURE, "STRUCT_TYPE_FIGURE", "figure"}, {STRUCT_TYPE_NB_TYPES, "STRUCT_TYPE_NB_TYPES", "nb-types"}, {0, NULL, NULL}};
		GType struct_type_type_id;
		struct_type_type_id = g_enum_register_static ("StructType", values);
		g_once_init_leave (&struct_type_type_id__volatile, struct_type_type_id);
	}
	return struct_type_type_id__volatile;
}


GType struct_action_get_type (void) {
	static volatile gsize struct_action_type_id__volatile = 0;
	if (g_once_init_enter (&struct_action_type_id__volatile)) {
		static const GEnumValue values[] = {{STRUCT_ACTION_CUT, "STRUCT_ACTION_CUT", "cut"}, {STRUCT_ACTION_COPY, "STRUCT_ACTION_COPY", "copy"}, {STRUCT_ACTION_DELETE, "STRUCT_ACTION_DELETE", "delete"}, {STRUCT_ACTION_SELECT, "STRUCT_ACTION_SELECT", "select"}, {STRUCT_ACTION_COMMENT, "STRUCT_ACTION_COMMENT", "comment"}, {STRUCT_ACTION_SHIFT_LEFT, "STRUCT_ACTION_SHIFT_LEFT", "shift-left"}, {STRUCT_ACTION_SHIFT_RIGHT, "STRUCT_ACTION_SHIFT_RIGHT", "shift-right"}, {STRUCT_ACTION_NB_ACTIONS, "STRUCT_ACTION_NB_ACTIONS", "nb-actions"}, {0, NULL, NULL}};
		GType struct_action_type_id;
		struct_action_type_id = g_enum_register_static ("StructAction", values);
		g_once_init_leave (&struct_action_type_id__volatile, struct_action_type_id);
	}
	return struct_action_type_id__volatile;
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


static void _structure_connect_parsing_gtk_widget_show (GtkWidget* _sender, gpointer self) {
	structure_connect_parsing (self);
}


static void __lambda130_ (Structure* self) {
	GtkAction* _tmp0_;
	structure_disconnect_parsing (self);
	_tmp0_ = self->priv->_action_all_menu;
	gtk_action_set_sensitive (_tmp0_, FALSE);
}


static void ___lambda130__gtk_widget_hide (GtkWidget* _sender, gpointer self) {
	__lambda130_ (self);
}


Structure* structure_construct (GType object_type, MainWindow* main_window, GtkUIManager* ui_manager) {
	Structure * self = NULL;
	MainWindow* _tmp0_;
	GtkUIManager* _tmp1_;
	GtkWidget* _tmp2_ = NULL;
	GtkMenu* _tmp3_;
	GtkUIManager* _tmp4_;
	GtkAction* _tmp5_ = NULL;
	GtkAction* _tmp6_;
	GtkUIManager* _tmp7_;
	GtkAction* _tmp8_ = NULL;
	GtkAction* _tmp9_;
	GtkUIManager* _tmp10_;
	GtkAction* _tmp11_ = NULL;
	GtkAction* _tmp12_;
	GtkUIManager* _tmp13_;
	GtkAction* _tmp14_ = NULL;
	GtkAction* _tmp15_;
	GtkUIManager* _tmp16_;
	GtkAction* _tmp17_ = NULL;
	GtkAction* _tmp18_;
	GtkUIManager* _tmp19_;
	GtkAction* _tmp20_ = NULL;
	GtkAction* _tmp21_;
	GtkUIManager* _tmp22_;
	GtkAction* _tmp23_ = NULL;
	GtkAction* _tmp24_;
	GtkUIManager* _tmp25_;
	GtkAction* _tmp26_ = NULL;
	GtkAction* _tmp27_;
	GtkWidget* _tmp28_;
	g_return_val_if_fail (main_window != NULL, NULL);
	g_return_val_if_fail (ui_manager != NULL, NULL);
	self = (Structure*) g_object_new (object_type, "spacing", 3, NULL);
	_tmp0_ = main_window;
	self->priv->_main_window = _tmp0_;
	_tmp1_ = ui_manager;
	_tmp2_ = gtk_ui_manager_get_widget (_tmp1_, "/StructurePopup");
	_tmp3_ = _g_object_ref0 (GTK_IS_MENU (_tmp2_) ? ((GtkMenu*) _tmp2_) : NULL);
	_g_object_unref0 (self->priv->_popup_menu);
	self->priv->_popup_menu = _tmp3_;
	_tmp4_ = ui_manager;
	_tmp5_ = gtk_ui_manager_get_action (_tmp4_, "/MainMenu/Structure");
	_tmp6_ = _g_object_ref0 (_tmp5_);
	_g_object_unref0 (self->priv->_action_all_menu);
	self->priv->_action_all_menu = _tmp6_;
	_tmp7_ = ui_manager;
	_tmp8_ = gtk_ui_manager_get_action (_tmp7_, "/StructurePopup/StructureCut");
	_tmp9_ = _g_object_ref0 (_tmp8_);
	_g_object_unref0 (self->priv->_action_cut);
	self->priv->_action_cut = _tmp9_;
	_tmp10_ = ui_manager;
	_tmp11_ = gtk_ui_manager_get_action (_tmp10_, "/StructurePopup/StructureCopy");
	_tmp12_ = _g_object_ref0 (_tmp11_);
	_g_object_unref0 (self->priv->_action_copy);
	self->priv->_action_copy = _tmp12_;
	_tmp13_ = ui_manager;
	_tmp14_ = gtk_ui_manager_get_action (_tmp13_, "/StructurePopup/StructureDelete");
	_tmp15_ = _g_object_ref0 (_tmp14_);
	_g_object_unref0 (self->priv->_action_delete);
	self->priv->_action_delete = _tmp15_;
	_tmp16_ = ui_manager;
	_tmp17_ = gtk_ui_manager_get_action (_tmp16_, "/StructurePopup/StructureSelect");
	_tmp18_ = _g_object_ref0 (_tmp17_);
	_g_object_unref0 (self->priv->_action_select);
	self->priv->_action_select = _tmp18_;
	_tmp19_ = ui_manager;
	_tmp20_ = gtk_ui_manager_get_action (_tmp19_, "/StructurePopup/StructureComment");
	_tmp21_ = _g_object_ref0 (_tmp20_);
	_g_object_unref0 (self->priv->_action_comment);
	self->priv->_action_comment = _tmp21_;
	_tmp22_ = ui_manager;
	_tmp23_ = gtk_ui_manager_get_action (_tmp22_, "/StructurePopup/StructureShiftLeft");
	_tmp24_ = _g_object_ref0 (_tmp23_);
	_g_object_unref0 (self->priv->_action_shift_left);
	self->priv->_action_shift_left = _tmp24_;
	_tmp25_ = ui_manager;
	_tmp26_ = gtk_ui_manager_get_action (_tmp25_, "/StructurePopup/StructureShiftRight");
	_tmp27_ = _g_object_ref0 (_tmp26_);
	_g_object_unref0 (self->priv->_action_shift_right);
	self->priv->_action_shift_right = _tmp27_;
	structure_init_toolbar (self);
	structure_init_vpaned (self);
	structure_init_list_view (self);
	structure_init_tree_view (self);
	gtk_widget_show_all ((GtkWidget*) self);
	_tmp28_ = self->priv->_list_view_sw;
	gtk_widget_hide (_tmp28_);
	g_signal_connect_object ((GtkWidget*) self, "show", (GCallback) _structure_connect_parsing_gtk_widget_show, self, 0);
	g_signal_connect_object ((GtkWidget*) self, "hide", (GCallback) ___lambda130__gtk_widget_hide, self, 0);
	return self;
}


Structure* structure_new (MainWindow* main_window, GtkUIManager* ui_manager) {
	return structure_construct (TYPE_STRUCTURE, main_window, ui_manager);
}


static void __lambda119_ (Structure* self) {
	MainWindow* _tmp0_;
	Document* _tmp1_;
	Document* _tmp2_;
	_tmp0_ = self->priv->_main_window;
	_tmp1_ = main_window_get_active_document (_tmp0_);
	_tmp2_ = _tmp1_;
	structure_show_document (self, _tmp2_, TRUE);
}


static void ___lambda119__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	__lambda119_ (self);
}


static void __lambda121_ (Structure* self) {
	GtkTreeView* _tmp0_;
	_tmp0_ = self->priv->_tree_view;
	gtk_tree_view_expand_all (_tmp0_);
}


static void ___lambda121__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	__lambda121_ (self);
}


static void __lambda122_ (Structure* self) {
	GtkTreeView* _tmp0_;
	_tmp0_ = self->priv->_tree_view;
	gtk_tree_view_collapse_all (_tmp0_);
}


static void ___lambda122__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	__lambda122_ (self);
}


static void structure_init_toolbar (Structure* self) {
	GtkHBox* _tmp0_;
	GtkHBox* _tmp1_;
	GtkHBox* hbox;
	GtkButton* _tmp2_ = NULL;
	GtkButton* refresh_button;
	const gchar* _tmp3_ = NULL;
	GtkSeparatorToolItem* _tmp4_;
	GtkSeparatorToolItem* _tmp5_;
	GtkSeparatorToolItem* sep;
	GtkSeparatorToolItem* _tmp6_;
	GtkButton* _tmp7_ = NULL;
	GtkButton* expand_button;
	const gchar* _tmp8_ = NULL;
	GtkButton* _tmp9_ = NULL;
	GtkButton* collapse_button;
	const gchar* _tmp10_ = NULL;
	GtkSeparatorToolItem* _tmp11_;
	GtkSeparatorToolItem* _tmp12_;
	GtkSeparatorToolItem* _tmp13_;
	StructType* _tmp14_ = NULL;
	StructType* _tmp15_;
	gint _tmp15__length1;
	const gchar* _tmp16_ = NULL;
	GtkToggleButton* _tmp17_ = NULL;
	GtkToggleButton* _tmp18_;
	GtkToggleButton* toggle_button;
	GtkToggleButton* _tmp19_;
	StructType* _tmp20_ = NULL;
	StructType* _tmp21_;
	gint _tmp21__length1;
	const gchar* _tmp22_ = NULL;
	GtkToggleButton* _tmp23_ = NULL;
	GtkToggleButton* _tmp24_;
	StructType* _tmp25_ = NULL;
	StructType* _tmp26_;
	gint _tmp26__length1;
	const gchar* _tmp27_ = NULL;
	GtkToggleButton* _tmp28_ = NULL;
	GtkToggleButton* _tmp29_;
	StructType* _tmp30_ = NULL;
	StructType* _tmp31_;
	gint _tmp31__length1;
	const gchar* _tmp32_ = NULL;
	GtkToggleButton* _tmp33_ = NULL;
	GtkToggleButton* _tmp34_;
	StructType* _tmp35_ = NULL;
	StructType* _tmp36_;
	gint _tmp36__length1;
	const gchar* _tmp37_ = NULL;
	GtkToggleButton* _tmp38_ = NULL;
	GtkToggleButton* _tmp39_;
	g_return_if_fail (self != NULL);
	_tmp0_ = (GtkHBox*) gtk_hbox_new (FALSE, 0);
	_tmp1_ = g_object_ref_sink (_tmp0_);
	hbox = _tmp1_;
	gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) hbox, FALSE, FALSE, (guint) 0);
	_tmp2_ = utils_get_toolbar_button (GTK_STOCK_REFRESH);
	refresh_button = _tmp2_;
	_tmp3_ = _ ("Refresh");
	gtk_widget_set_tooltip_text ((GtkWidget*) refresh_button, _tmp3_);
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) refresh_button, TRUE, TRUE, (guint) 0);
	g_signal_connect_object (refresh_button, "clicked", (GCallback) ___lambda119__gtk_button_clicked, self, 0);
	_tmp4_ = (GtkSeparatorToolItem*) gtk_separator_tool_item_new ();
	_tmp5_ = g_object_ref_sink (_tmp4_);
	sep = _tmp5_;
	_tmp6_ = sep;
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) _tmp6_, FALSE, TRUE, (guint) 0);
	_tmp7_ = utils_get_toolbar_button (GTK_STOCK_ZOOM_IN);
	expand_button = _tmp7_;
	_tmp8_ = _ ("Expand All");
	gtk_widget_set_tooltip_text ((GtkWidget*) expand_button, _tmp8_);
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) expand_button, TRUE, TRUE, (guint) 0);
	g_signal_connect_object (expand_button, "clicked", (GCallback) ___lambda121__gtk_button_clicked, self, 0);
	_tmp9_ = utils_get_toolbar_button (GTK_STOCK_ZOOM_OUT);
	collapse_button = _tmp9_;
	_tmp10_ = _ ("Collapse All");
	gtk_widget_set_tooltip_text ((GtkWidget*) collapse_button, _tmp10_);
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) collapse_button, TRUE, TRUE, (guint) 0);
	g_signal_connect_object (collapse_button, "clicked", (GCallback) ___lambda122__gtk_button_clicked, self, 0);
	_tmp11_ = (GtkSeparatorToolItem*) gtk_separator_tool_item_new ();
	_tmp12_ = g_object_ref_sink (_tmp11_);
	_g_object_unref0 (sep);
	sep = _tmp12_;
	_tmp13_ = sep;
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) _tmp13_, FALSE, TRUE, (guint) 0);
	_tmp14_ = g_new0 (StructType, 1);
	_tmp14_[0] = STRUCT_TYPE_LABEL;
	_tmp15_ = _tmp14_;
	_tmp15__length1 = 1;
	_tmp16_ = _ ("Show labels");
	_tmp17_ = structure_create_simple_list_button (self, _tmp15_, 1, _tmp16_);
	_tmp18_ = _tmp17_;
	_tmp15_ = (g_free (_tmp15_), NULL);
	toggle_button = _tmp18_;
	_tmp19_ = toggle_button;
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) _tmp19_, TRUE, TRUE, (guint) 0);
	_tmp20_ = g_new0 (StructType, 1);
	_tmp20_[0] = STRUCT_TYPE_INCLUDE;
	_tmp21_ = _tmp20_;
	_tmp21__length1 = 1;
	_tmp22_ = _ ("Show files included");
	_tmp23_ = structure_create_simple_list_button (self, _tmp21_, 1, _tmp22_);
	_g_object_unref0 (toggle_button);
	toggle_button = _tmp23_;
	_tmp21_ = (g_free (_tmp21_), NULL);
	_tmp24_ = toggle_button;
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) _tmp24_, TRUE, TRUE, (guint) 0);
	_tmp25_ = g_new0 (StructType, 1);
	_tmp25_[0] = STRUCT_TYPE_TABLE;
	_tmp26_ = _tmp25_;
	_tmp26__length1 = 1;
	_tmp27_ = _ ("Show tables");
	_tmp28_ = structure_create_simple_list_button (self, _tmp26_, 1, _tmp27_);
	_g_object_unref0 (toggle_button);
	toggle_button = _tmp28_;
	_tmp26_ = (g_free (_tmp26_), NULL);
	_tmp29_ = toggle_button;
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) _tmp29_, TRUE, TRUE, (guint) 0);
	_tmp30_ = g_new0 (StructType, 2);
	_tmp30_[0] = STRUCT_TYPE_FIGURE;
	_tmp30_[1] = STRUCT_TYPE_IMAGE;
	_tmp31_ = _tmp30_;
	_tmp31__length1 = 2;
	_tmp32_ = _ ("Show figures and images");
	_tmp33_ = structure_create_simple_list_button (self, _tmp31_, 2, _tmp32_);
	_g_object_unref0 (toggle_button);
	toggle_button = _tmp33_;
	_tmp31_ = (g_free (_tmp31_), NULL);
	_tmp34_ = toggle_button;
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) _tmp34_, TRUE, TRUE, (guint) 0);
	_tmp35_ = g_new0 (StructType, 2);
	_tmp35_[0] = STRUCT_TYPE_TODO;
	_tmp35_[1] = STRUCT_TYPE_FIXME;
	_tmp36_ = _tmp35_;
	_tmp36__length1 = 2;
	_tmp37_ = _ ("Show TODOs and FIXMEs");
	_tmp38_ = structure_create_simple_list_button (self, _tmp36_, 2, _tmp37_);
	_g_object_unref0 (toggle_button);
	toggle_button = _tmp38_;
	_tmp36_ = (g_free (_tmp36_), NULL);
	_tmp39_ = toggle_button;
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) _tmp39_, TRUE, TRUE, (guint) 0);
	_g_object_unref0 (toggle_button);
	_g_object_unref0 (collapse_button);
	_g_object_unref0 (expand_button);
	_g_object_unref0 (sep);
	_g_object_unref0 (refresh_button);
	_g_object_unref0 (hbox);
}


static Block21Data* block21_data_ref (Block21Data* _data21_) {
	g_atomic_int_inc (&_data21_->_ref_count_);
	return _data21_;
}


static void block21_data_unref (Block21Data* _data21_) {
	if (g_atomic_int_dec_and_test (&_data21_->_ref_count_)) {
		Structure * self;
		self = _data21_->self;
		_g_object_unref0 (_data21_->button);
		_g_object_unref0 (self);
		g_slice_free (Block21Data, _data21_);
	}
}


static void _vala_array_add15 (GtkToggleButton*** array, int* length, int* size, GtkToggleButton* value) {
	if ((*length) == (*size)) {
		*size = (*size) ? (2 * (*size)) : 4;
		*array = g_renew (GtkToggleButton*, *array, (*size) + 1);
	}
	(*array)[(*length)++] = value;
	(*array)[*length] = NULL;
}


static gboolean _vala_struct_type_array_contains (StructType* stack, int stack_length, StructType needle) {
	int i;
	for (i = 0; i < stack_length; i++) {
		if (needle == stack[i]) {
			return TRUE;
		}
	}
	return FALSE;
}


static StructType* _vala_array_dup3 (StructType* self, int length) {
	return g_memdup (self, length * sizeof (StructType));
}


static void __lambda123_ (Block21Data* _data21_) {
	Structure * self;
	gboolean _tmp0_ = FALSE;
	StructType* _tmp6_;
	gint _tmp6__length1;
	StructType* _tmp7_;
	gint _tmp7__length1;
	GtkWidget* _tmp8_;
	GtkToggleButton** _tmp9_;
	gint _tmp9__length1;
	self = _data21_->self;
	_tmp0_ = gtk_toggle_button_get_active (_data21_->button);
	if (!_tmp0_) {
		gboolean _tmp1_ = FALSE;
		gboolean _tmp2_;
		gboolean _tmp4_;
		_tmp2_ = self->priv->_list_is_hidden;
		if (!_tmp2_) {
			StructType* _tmp3_;
			gint _tmp3__length1;
			_tmp3_ = self->priv->_current_list_types;
			_tmp3__length1 = self->priv->_current_list_types_length1;
			_tmp1_ = _vala_struct_type_array_contains (_tmp3_, _tmp3__length1, _data21_->main_type);
		} else {
			_tmp1_ = FALSE;
		}
		_tmp4_ = _tmp1_;
		if (_tmp4_) {
			GtkWidget* _tmp5_;
			self->priv->_list_is_hidden = TRUE;
			_tmp5_ = self->priv->_list_view_sw;
			gtk_widget_hide (_tmp5_);
		}
		return;
	}
	_tmp6_ = _data21_->types;
	_tmp6__length1 = _data21_->types_length1;
	_tmp7_ = (_tmp6_ != NULL) ? _vala_array_dup3 (_tmp6_, _tmp6__length1) : ((gpointer) _tmp6_);
	_tmp7__length1 = _tmp6__length1;
	self->priv->_current_list_types = (g_free (self->priv->_current_list_types), NULL);
	self->priv->_current_list_types = _tmp7_;
	self->priv->_current_list_types_length1 = _tmp7__length1;
	self->priv->__current_list_types_size_ = self->priv->_current_list_types_length1;
	self->priv->_list_is_hidden = FALSE;
	_tmp8_ = self->priv->_list_view_sw;
	gtk_widget_show_all (_tmp8_);
	structure_populate_simple_list (self);
	_tmp9_ = self->priv->_simple_list_buttons;
	_tmp9__length1 = self->priv->_simple_list_buttons_length1;
	{
		GtkToggleButton** simple_list_button_collection = NULL;
		gint simple_list_button_collection_length1 = 0;
		gint _simple_list_button_collection_size_ = 0;
		gint simple_list_button_it = 0;
		simple_list_button_collection = _tmp9_;
		simple_list_button_collection_length1 = _tmp9__length1;
		for (simple_list_button_it = 0; simple_list_button_it < _tmp9__length1; simple_list_button_it = simple_list_button_it + 1) {
			GtkToggleButton* _tmp10_;
			GtkToggleButton* simple_list_button = NULL;
			_tmp10_ = _g_object_ref0 (simple_list_button_collection[simple_list_button_it]);
			simple_list_button = _tmp10_;
			{
				GtkToggleButton* _tmp11_;
				GtkToggleButton* _tmp12_;
				_tmp11_ = simple_list_button;
				if (_tmp11_ == _data21_->button) {
					_g_object_unref0 (simple_list_button);
					continue;
				}
				_tmp12_ = simple_list_button;
				gtk_toggle_button_set_active (_tmp12_, FALSE);
				_g_object_unref0 (simple_list_button);
			}
		}
	}
}


static void ___lambda123__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	__lambda123_ (self);
}


static GtkToggleButton* structure_create_simple_list_button (Structure* self, StructType* types, int types_length1, const gchar* tooltip) {
	GtkToggleButton* result = NULL;
	Block21Data* _data21_;
	StructType* _tmp0_;
	gint _tmp0__length1;
	StructType* _tmp1_;
	gint _tmp1__length1;
	StructType* _tmp2_;
	gint _tmp2__length1;
	StructType _tmp3_;
	gchar* _tmp4_ = NULL;
	gchar* _tmp5_;
	GtkToggleButton* _tmp6_ = NULL;
	GtkToggleButton* _tmp7_;
	const gchar* _tmp8_;
	GtkToggleButton** _tmp9_;
	gint _tmp9__length1;
	GtkToggleButton* _tmp10_;
	GtkToggleButton* _tmp11_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (tooltip != NULL, NULL);
	_data21_ = g_slice_new0 (Block21Data);
	_data21_->_ref_count_ = 1;
	_data21_->self = g_object_ref (self);
	_tmp0_ = types;
	_tmp0__length1 = types_length1;
	_data21_->types = _tmp0_;
	_data21_->types_length1 = _tmp0__length1;
	_tmp1_ = _data21_->types;
	_tmp1__length1 = _data21_->types_length1;
	g_return_val_if_fail (_tmp1__length1 > 0, NULL);
	_tmp2_ = _data21_->types;
	_tmp2__length1 = _data21_->types_length1;
	_tmp3_ = _tmp2_[0];
	_data21_->main_type = _tmp3_;
	_tmp4_ = structure_get_icon_from_type (_data21_->main_type);
	_tmp5_ = _tmp4_;
	_tmp6_ = utils_get_toolbar_toggle_button (_tmp5_);
	_tmp7_ = _tmp6_;
	_g_free0 (_tmp5_);
	_data21_->button = _tmp7_;
	_tmp8_ = tooltip;
	gtk_widget_set_tooltip_text ((GtkWidget*) _data21_->button, _tmp8_);
	_tmp9_ = self->priv->_simple_list_buttons;
	_tmp9__length1 = self->priv->_simple_list_buttons_length1;
	_tmp10_ = _g_object_ref0 (_data21_->button);
	_vala_array_add15 (&self->priv->_simple_list_buttons, &self->priv->_simple_list_buttons_length1, &self->priv->__simple_list_buttons_size_, _tmp10_);
	g_signal_connect_data ((GtkButton*) _data21_->button, "clicked", (GCallback) ___lambda123__gtk_button_clicked, block21_data_ref (_data21_), (GClosureNotify) block21_data_unref, 0);
	_tmp11_ = _g_object_ref0 (_data21_->button);
	result = _tmp11_;
	block21_data_unref (_data21_);
	_data21_ = NULL;
	return result;
}


static void _gtk_tree_path_free0_ (gpointer var) {
	(var == NULL) ? NULL : (var = (gtk_tree_path_free (var), NULL));
}


static void _g_list_free__gtk_tree_path_free0_ (GList* self) {
	g_list_foreach (self, (GFunc) _gtk_tree_path_free0_, NULL);
	g_list_free (self);
}


static gpointer _gtk_tree_path_copy0 (gpointer self) {
	return self ? gtk_tree_path_copy (self) : NULL;
}


static void structure_populate_simple_list (Structure* self) {
	GtkListStore* _tmp0_;
	gboolean _tmp1_ = FALSE;
	StructureModel* _tmp2_;
	gboolean _tmp4_;
	StructureModel* _tmp5_;
	GtkListStore* _tmp6_;
	StructType* _tmp7_;
	gint _tmp7__length1;
	StructType _tmp8_;
	GtkTreeView* _tmp9_;
	GtkTreeSelection* _tmp10_ = NULL;
	GtkTreeSelection* _tmp11_;
	GtkTreeSelection* tree_select;
	GtkTreeSelection* _tmp12_;
	GList* _tmp13_ = NULL;
	GList* selected_rows;
	GList* _tmp14_;
	guint _tmp15_ = 0U;
	GList* _tmp16_;
	gconstpointer _tmp17_ = NULL;
	GtkTreePath* _tmp18_;
	GtkTreePath* tree_path;
	GtkTreeIter tree_iter = {0};
	StructureModel* _tmp19_;
	GtkTreePath* _tmp20_;
	GtkTreeIter _tmp21_ = {0};
	gboolean _tmp22_ = FALSE;
	GtkTreeIter _tmp23_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->_list_store;
	gtk_list_store_clear (_tmp0_);
	_tmp2_ = self->priv->_model;
	if (_tmp2_ == NULL) {
		_tmp1_ = TRUE;
	} else {
		gboolean _tmp3_;
		_tmp3_ = self->priv->_list_is_hidden;
		_tmp1_ = _tmp3_;
	}
	_tmp4_ = _tmp1_;
	if (_tmp4_) {
		return;
	}
	_tmp5_ = self->priv->_model;
	_tmp6_ = self->priv->_list_store;
	_tmp7_ = self->priv->_current_list_types;
	_tmp7__length1 = self->priv->_current_list_types_length1;
	_tmp8_ = _tmp7_[0];
	structure_model_populate_list (_tmp5_, _tmp6_, _tmp8_);
	_tmp9_ = self->priv->_tree_view;
	_tmp10_ = gtk_tree_view_get_selection (_tmp9_);
	_tmp11_ = _g_object_ref0 (_tmp10_);
	tree_select = _tmp11_;
	_tmp12_ = tree_select;
	_tmp13_ = gtk_tree_selection_get_selected_rows (_tmp12_, NULL);
	selected_rows = _tmp13_;
	_tmp14_ = selected_rows;
	_tmp15_ = g_list_length (_tmp14_);
	if (_tmp15_ != ((guint) 1)) {
		__g_list_free__gtk_tree_path_free0_0 (selected_rows);
		_g_object_unref0 (tree_select);
		return;
	}
	_tmp16_ = selected_rows;
	_tmp17_ = g_list_nth_data (_tmp16_, (guint) 0);
	_tmp18_ = _gtk_tree_path_copy0 ((GtkTreePath*) _tmp17_);
	tree_path = _tmp18_;
	_tmp19_ = self->priv->_model;
	_tmp20_ = tree_path;
	_tmp22_ = gtk_tree_model_get_iter ((GtkTreeModel*) _tmp19_, &_tmp21_, _tmp20_);
	tree_iter = _tmp21_;
	if (!_tmp22_) {
		g_return_if_reached ();
	}
	_tmp23_ = tree_iter;
	structure_select_simple_list_item (self, &_tmp23_);
	_gtk_tree_path_free0 (tree_path);
	__g_list_free__gtk_tree_path_free0_0 (selected_rows);
	_g_object_unref0 (tree_select);
}


static void structure_init_vpaned (Structure* self) {
	GtkVPaned* _tmp0_;
	GtkVPaned* _tmp1_;
	GtkVPaned* _tmp2_;
	GSettings* _tmp3_;
	GSettings* settings;
	GtkVPaned* _tmp4_;
	gint _tmp5_ = 0;
	g_return_if_fail (self != NULL);
	_tmp0_ = (GtkVPaned*) gtk_vpaned_new ();
	_tmp1_ = g_object_ref_sink (_tmp0_);
	_g_object_unref0 (self->priv->_vpaned);
	self->priv->_vpaned = _tmp1_;
	_tmp2_ = self->priv->_vpaned;
	gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) _tmp2_, TRUE, TRUE, (guint) 0);
	_tmp3_ = g_settings_new ("org.gnome.latexila.state.window");
	settings = _tmp3_;
	_tmp4_ = self->priv->_vpaned;
	_tmp5_ = g_settings_get_int (settings, "structure-paned-position");
	gtk_paned_set_position ((GtkPaned*) _tmp4_, _tmp5_);
	_g_object_unref0 (settings);
}


void structure_save_state (Structure* self) {
	GSettings* _tmp0_;
	GSettings* settings;
	GtkVPaned* _tmp1_;
	gint _tmp2_ = 0;
	g_return_if_fail (self != NULL);
	_tmp0_ = g_settings_new ("org.gnome.latexila.state.window");
	settings = _tmp0_;
	_tmp1_ = self->priv->_vpaned;
	_tmp2_ = gtk_paned_get_position ((GtkPaned*) _tmp1_);
	g_settings_set_int (settings, "structure-paned-position", _tmp2_);
	_g_object_unref0 (settings);
}


static gboolean __lambda124_ (Structure* self, GtkTreeSelection* select, GtkTreeModel* model, GtkTreePath* path, gboolean path_currently_selected) {
	gboolean result = FALSE;
	gboolean _tmp0_;
	GtkTreePath* _tmp1_;
	gboolean _tmp2_ = FALSE;
	g_return_val_if_fail (select != NULL, FALSE);
	g_return_val_if_fail (model != NULL, FALSE);
	g_return_val_if_fail (path != NULL, FALSE);
	_tmp0_ = path_currently_selected;
	if (_tmp0_) {
		result = TRUE;
		return result;
	}
	_tmp1_ = path;
	_tmp2_ = structure_select_list_row (self, _tmp1_);
	result = _tmp2_;
	return result;
}


static gboolean ___lambda124__gtk_tree_selection_func (GtkTreeSelection* selection, GtkTreeModel* model, GtkTreePath* path, gboolean path_currently_selected, gpointer self) {
	gboolean result;
	result = __lambda124_ (self, selection, model, path, path_currently_selected);
	return result;
}


static void __lambda125_ (Structure* self, GtkTreePath* path) {
	GtkTreePath* _tmp0_;
	g_return_if_fail (path != NULL);
	_tmp0_ = path;
	structure_select_list_row (self, _tmp0_);
}


static void ___lambda125__gtk_tree_view_row_activated (GtkTreeView* _sender, GtkTreePath* path, GtkTreeViewColumn* column, gpointer self) {
	__lambda125_ (self, path);
}


static void structure_init_list_view (Structure* self) {
	GtkTreeView* _tmp0_ = NULL;
	GtkListStore* _tmp1_;
	GtkTreeView* _tmp2_;
	GtkListStore* _tmp3_;
	GtkTreeView* _tmp4_;
	GtkTreeSelection* _tmp5_ = NULL;
	GtkTreeSelection* _tmp6_;
	GtkTreeSelection* list_select;
	GtkTreeView* _tmp7_;
	GtkTreeView* _tmp8_;
	GtkWidget* _tmp9_ = NULL;
	GtkVPaned* _tmp10_;
	GtkWidget* _tmp11_;
	g_return_if_fail (self != NULL);
	_tmp0_ = structure_get_new_tree_view (self, (gint) STRUCT_LIST_COLUMN_PIXBUF, (gint) STRUCT_LIST_COLUMN_TEXT, (gint) STRUCT_LIST_COLUMN_TOOLTIP);
	_g_object_unref0 (self->priv->_list_view);
	self->priv->_list_view = _tmp0_;
	_tmp1_ = gtk_list_store_new ((gint) STRUCT_LIST_COLUMN_N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	_g_object_unref0 (self->priv->_list_store);
	self->priv->_list_store = _tmp1_;
	_tmp2_ = self->priv->_list_view;
	_tmp3_ = self->priv->_list_store;
	gtk_tree_view_set_model (_tmp2_, (GtkTreeModel*) _tmp3_);
	_tmp4_ = self->priv->_list_view;
	_tmp5_ = gtk_tree_view_get_selection (_tmp4_);
	_tmp6_ = _g_object_ref0 (_tmp5_);
	list_select = _tmp6_;
	gtk_tree_selection_set_select_function (list_select, ___lambda124__gtk_tree_selection_func, g_object_ref (self), g_object_unref);
	_tmp7_ = self->priv->_list_view;
	g_signal_connect_object (_tmp7_, "row-activated", (GCallback) ___lambda125__gtk_tree_view_row_activated, self, 0);
	_tmp8_ = self->priv->_list_view;
	_tmp9_ = utils_add_scrollbar ((GtkWidget*) _tmp8_);
	_g_object_unref0 (self->priv->_list_view_sw);
	self->priv->_list_view_sw = _tmp9_;
	_tmp10_ = self->priv->_vpaned;
	_tmp11_ = self->priv->_list_view_sw;
	gtk_paned_add1 ((GtkPaned*) _tmp10_, _tmp11_);
	_g_object_unref0 (list_select);
}


static gboolean __lambda126_ (Structure* self, GtkTreeSelection* select, GtkTreeModel* model, GtkTreePath* path, gboolean path_currently_selected) {
	gboolean result = FALSE;
	gboolean _tmp0_;
	GtkTreePath* _tmp1_;
	gboolean _tmp2_ = FALSE;
	g_return_val_if_fail (select != NULL, FALSE);
	g_return_val_if_fail (model != NULL, FALSE);
	g_return_val_if_fail (path != NULL, FALSE);
	_tmp0_ = path_currently_selected;
	if (_tmp0_) {
		result = TRUE;
		return result;
	}
	_tmp1_ = path;
	_tmp2_ = structure_select_tree_row (self, _tmp1_);
	result = _tmp2_;
	return result;
}


static gboolean ___lambda126__gtk_tree_selection_func (GtkTreeSelection* selection, GtkTreeModel* model, GtkTreePath* path, gboolean path_currently_selected, gpointer self) {
	gboolean result;
	result = __lambda126_ (self, selection, model, path, path_currently_selected);
	return result;
}


static void __lambda127_ (Structure* self, GtkTreePath* path) {
	GtkTreePath* _tmp0_;
	g_return_if_fail (path != NULL);
	_tmp0_ = path;
	structure_select_tree_row (self, _tmp0_);
}


static void ___lambda127__gtk_tree_view_row_activated (GtkTreeView* _sender, GtkTreePath* path, GtkTreeViewColumn* column, gpointer self) {
	__lambda127_ (self, path);
}


static gboolean __lambda128_ (Structure* self, GdkEventButton* event) {
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
	GdkEventButton _tmp1_;
	guint _tmp2_;
	gboolean _tmp5_;
	g_return_val_if_fail (event != NULL, FALSE);
	_tmp1_ = *event;
	_tmp2_ = _tmp1_.button;
	if (_tmp2_ == ((guint) 3)) {
		GdkEventButton _tmp3_;
		GdkEventType _tmp4_;
		_tmp3_ = *event;
		_tmp4_ = _tmp3_.type;
		_tmp0_ = _tmp4_ == GDK_BUTTON_PRESS;
	} else {
		_tmp0_ = FALSE;
	}
	_tmp5_ = _tmp0_;
	if (_tmp5_) {
		GdkEventButton _tmp6_;
		_tmp6_ = *event;
		structure_show_popup_menu (self, &_tmp6_);
	}
	result = FALSE;
	return result;
}


static gboolean ___lambda128__gtk_widget_button_press_event (GtkWidget* _sender, GdkEventButton* event, gpointer self) {
	gboolean result;
	result = __lambda128_ (self, event);
	return result;
}


static gboolean __lambda129_ (Structure* self) {
	gboolean result = FALSE;
	structure_show_popup_menu (self, NULL);
	result = TRUE;
	return result;
}


static gboolean ___lambda129__gtk_widget_popup_menu (GtkWidget* _sender, gpointer self) {
	gboolean result;
	result = __lambda129_ (self);
	return result;
}


static void structure_init_tree_view (Structure* self) {
	GtkTreeView* _tmp0_ = NULL;
	GtkTreeView* _tmp1_;
	GtkTreeView* _tmp2_;
	GtkTreeSelection* _tmp3_ = NULL;
	GtkTreeSelection* _tmp4_;
	GtkTreeSelection* tree_select;
	GtkTreeView* _tmp5_;
	GtkMenu* _tmp6_;
	GtkTreeView* _tmp7_;
	GtkTreeView* _tmp8_;
	GtkTreeView* _tmp9_;
	GtkTreeView* _tmp10_;
	GtkWidget* _tmp11_ = NULL;
	GtkWidget* sw;
	GtkVPaned* _tmp12_;
	g_return_if_fail (self != NULL);
	_tmp0_ = structure_get_new_tree_view (self, (gint) STRUCT_COLUMN_PIXBUF, (gint) STRUCT_COLUMN_TEXT, (gint) STRUCT_COLUMN_TOOLTIP);
	_g_object_unref0 (self->priv->_tree_view);
	self->priv->_tree_view = _tmp0_;
	_tmp1_ = self->priv->_tree_view;
	gtk_tree_view_set_enable_tree_lines (_tmp1_, TRUE);
	_tmp2_ = self->priv->_tree_view;
	_tmp3_ = gtk_tree_view_get_selection (_tmp2_);
	_tmp4_ = _g_object_ref0 (_tmp3_);
	tree_select = _tmp4_;
	gtk_tree_selection_set_select_function (tree_select, ___lambda126__gtk_tree_selection_func, g_object_ref (self), g_object_unref);
	_tmp5_ = self->priv->_tree_view;
	g_signal_connect_object (_tmp5_, "row-activated", (GCallback) ___lambda127__gtk_tree_view_row_activated, self, 0);
	_tmp6_ = self->priv->_popup_menu;
	_tmp7_ = self->priv->_tree_view;
	gtk_menu_attach_to_widget (_tmp6_, (GtkWidget*) _tmp7_, NULL);
	_tmp8_ = self->priv->_tree_view;
	g_signal_connect_object ((GtkWidget*) _tmp8_, "button-press-event", (GCallback) ___lambda128__gtk_widget_button_press_event, self, 0);
	_tmp9_ = self->priv->_tree_view;
	g_signal_connect_object ((GtkWidget*) _tmp9_, "popup-menu", (GCallback) ___lambda129__gtk_widget_popup_menu, self, 0);
	_tmp10_ = self->priv->_tree_view;
	_tmp11_ = utils_add_scrollbar ((GtkWidget*) _tmp10_);
	sw = _tmp11_;
	_tmp12_ = self->priv->_vpaned;
	gtk_paned_add2 ((GtkPaned*) _tmp12_, sw);
	_g_object_unref0 (sw);
	_g_object_unref0 (tree_select);
}


static GtkTreeView* structure_get_new_tree_view (Structure* self, gint pixbuf_col, gint text_col, gint tooltip_col) {
	GtkTreeView* result = NULL;
	GtkTreeView* _tmp0_;
	GtkTreeView* _tmp1_;
	GtkTreeView* tree_view;
	GtkTreeViewColumn* _tmp2_;
	GtkTreeViewColumn* _tmp3_;
	GtkTreeViewColumn* column;
	GtkCellRendererPixbuf* _tmp4_;
	GtkCellRendererPixbuf* _tmp5_;
	GtkCellRendererPixbuf* pixbuf_renderer;
	gint _tmp6_;
	GtkCellRendererText* _tmp7_;
	GtkCellRendererText* _tmp8_;
	GtkCellRendererText* text_renderer;
	gint _tmp9_;
	gint _tmp10_;
	GtkTreeSelection* _tmp11_ = NULL;
	GtkTreeSelection* _tmp12_;
	GtkTreeSelection* select;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = (GtkTreeView*) gtk_tree_view_new ();
	_tmp1_ = g_object_ref_sink (_tmp0_);
	tree_view = _tmp1_;
	gtk_tree_view_set_headers_visible (tree_view, FALSE);
	_tmp2_ = gtk_tree_view_column_new ();
	_tmp3_ = g_object_ref_sink (_tmp2_);
	column = _tmp3_;
	gtk_tree_view_append_column (tree_view, column);
	_tmp4_ = (GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ();
	_tmp5_ = g_object_ref_sink (_tmp4_);
	pixbuf_renderer = _tmp5_;
	gtk_cell_layout_pack_start ((GtkCellLayout*) column, (GtkCellRenderer*) pixbuf_renderer, FALSE);
	_tmp6_ = pixbuf_col;
	gtk_tree_view_column_set_attributes (column, (GtkCellRenderer*) pixbuf_renderer, "stock-id", _tmp6_, NULL, NULL);
	_tmp7_ = (GtkCellRendererText*) gtk_cell_renderer_text_new ();
	_tmp8_ = g_object_ref_sink (_tmp7_);
	text_renderer = _tmp8_;
	gtk_cell_layout_pack_start ((GtkCellLayout*) column, (GtkCellRenderer*) text_renderer, TRUE);
	_tmp9_ = text_col;
	gtk_tree_view_column_set_attributes (column, (GtkCellRenderer*) text_renderer, "text", _tmp9_, NULL, NULL);
	_tmp10_ = tooltip_col;
	gtk_tree_view_set_tooltip_column (tree_view, _tmp10_);
	_tmp11_ = gtk_tree_view_get_selection (tree_view);
	_tmp12_ = _g_object_ref0 (_tmp11_);
	select = _tmp12_;
	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
	result = tree_view;
	_g_object_unref0 (select);
	_g_object_unref0 (text_renderer);
	_g_object_unref0 (pixbuf_renderer);
	_g_object_unref0 (column);
	return result;
}


static gboolean structure_select_list_row (Structure* self, GtkTreePath* list_path) {
	gboolean result = FALSE;
	gboolean _tmp0_;
	StructureModel* _tmp1_;
	GtkTreeView* _tmp2_;
	GtkTreeSelection* _tmp3_ = NULL;
	GtkTreeSelection* _tmp4_;
	GtkTreeSelection* tree_select;
	GtkTreeSelection* _tmp5_;
	GtkTreePath* _tmp6_;
	gint* _tmp7_ = NULL;
	gint _tmp8_;
	gint row_num;
	StructureModel* _tmp9_;
	StructType* _tmp10_;
	gint _tmp10__length1;
	StructType _tmp11_;
	gint _tmp12_;
	GtkTreePath* _tmp13_ = NULL;
	GtkTreePath* tree_path;
	GtkTreePath* _tmp14_;
	GtkTreeView* _tmp15_;
	GtkTreePath* _tmp16_;
	GtkTreeSelection* _tmp17_;
	GtkTreePath* _tmp18_;
	GtkTreeView* _tmp19_;
	GtkTreePath* _tmp20_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (list_path != NULL, FALSE);
	_tmp0_ = self->priv->_first_select;
	if (!_tmp0_) {
		self->priv->_first_select = TRUE;
		result = TRUE;
		return result;
	}
	_tmp1_ = self->priv->_model;
	g_return_val_if_fail (_tmp1_ != NULL, FALSE);
	_tmp2_ = self->priv->_tree_view;
	_tmp3_ = gtk_tree_view_get_selection (_tmp2_);
	_tmp4_ = _g_object_ref0 (_tmp3_);
	tree_select = _tmp4_;
	_tmp5_ = tree_select;
	gtk_tree_selection_unselect_all (_tmp5_);
	_tmp6_ = list_path;
	_tmp7_ = gtk_tree_path_get_indices (_tmp6_);
	_tmp8_ = _tmp7_[0];
	row_num = _tmp8_;
	_tmp9_ = self->priv->_model;
	_tmp10_ = self->priv->_current_list_types;
	_tmp10__length1 = self->priv->_current_list_types_length1;
	_tmp11_ = _tmp10_[0];
	_tmp12_ = row_num;
	_tmp13_ = structure_model_get_tree_path_from_list_num (_tmp9_, _tmp11_, _tmp12_);
	tree_path = _tmp13_;
	_tmp14_ = tree_path;
	g_return_val_if_fail (_tmp14_ != NULL, FALSE);
	_tmp15_ = self->priv->_tree_view;
	_tmp16_ = tree_path;
	gtk_tree_view_expand_to_path (_tmp15_, _tmp16_);
	self->priv->_first_select = FALSE;
	_tmp17_ = tree_select;
	_tmp18_ = tree_path;
	gtk_tree_selection_select_path (_tmp17_, _tmp18_);
	_tmp19_ = self->priv->_tree_view;
	_tmp20_ = tree_path;
	gtk_tree_view_scroll_to_cell (_tmp19_, _tmp20_, NULL, TRUE, (gfloat) 0.5, (gfloat) 0);
	result = TRUE;
	_gtk_tree_path_free0 (tree_path);
	_g_object_unref0 (tree_select);
	return result;
}


static gboolean structure_select_tree_row (Structure* self, GtkTreePath* tree_path) {
	gboolean result = FALSE;
	gboolean _tmp0_;
	gboolean first_select;
	GtkTreeIter tree_iter = {0};
	StructureModel* _tmp1_;
	GtkTreePath* _tmp2_;
	GtkTreeIter _tmp3_ = {0};
	gboolean _tmp4_ = FALSE;
	GtkTextMark* mark = NULL;
	StructType type = 0;
	StructureModel* _tmp5_;
	GtkTreeIter _tmp6_;
	GtkTextMark* _tmp7_;
	GtkTextBuffer* _tmp8_ = NULL;
	GtkTextBuffer* _tmp9_;
	GtkTextBuffer* doc;
	GtkTextBuffer* _tmp10_;
	MainWindow* _tmp11_;
	Document* _tmp12_;
	Document* _tmp13_;
	GtkTextIter text_iter = {0};
	GtkTextBuffer* _tmp14_;
	GtkTextMark* _tmp15_;
	GtkTextIter _tmp16_ = {0};
	GtkTextBuffer* _tmp17_;
	GtkTextIter _tmp18_;
	MainWindow* _tmp19_;
	DocumentView* _tmp20_;
	DocumentView* _tmp21_;
	GtkTextBuffer* _tmp22_;
	GtkTextMark* _tmp23_ = NULL;
	StructType _tmp24_;
	GtkTreeIter _tmp25_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (tree_path != NULL, FALSE);
	_tmp0_ = self->priv->_first_select;
	first_select = _tmp0_;
	self->priv->_first_select = TRUE;
	_tmp1_ = self->priv->_model;
	_tmp2_ = tree_path;
	_tmp4_ = gtk_tree_model_get_iter ((GtkTreeModel*) _tmp1_, &_tmp3_, _tmp2_);
	tree_iter = _tmp3_;
	if (!_tmp4_) {
		g_return_val_if_reached (FALSE);
	}
	_tmp5_ = self->priv->_model;
	_tmp6_ = tree_iter;
	gtk_tree_model_get ((GtkTreeModel*) _tmp5_, &_tmp6_, STRUCT_COLUMN_START_MARK, &mark, STRUCT_COLUMN_TYPE, &type, -1, -1);
	_tmp7_ = mark;
	_tmp8_ = gtk_text_mark_get_buffer (_tmp7_);
	_tmp9_ = _g_object_ref0 (_tmp8_);
	doc = _tmp9_;
	_tmp10_ = doc;
	_tmp11_ = self->priv->_main_window;
	_tmp12_ = main_window_get_active_document (_tmp11_);
	_tmp13_ = _tmp12_;
	g_return_val_if_fail (_tmp10_ == GTK_TEXT_BUFFER (_tmp13_), FALSE);
	_tmp14_ = doc;
	_tmp15_ = mark;
	gtk_text_buffer_get_iter_at_mark (_tmp14_, &_tmp16_, _tmp15_);
	text_iter = _tmp16_;
	_tmp17_ = doc;
	_tmp18_ = text_iter;
	gtk_text_buffer_place_cursor (_tmp17_, &_tmp18_);
	_tmp19_ = self->priv->_main_window;
	_tmp20_ = main_window_get_active_view (_tmp19_);
	_tmp21_ = _tmp20_;
	_tmp22_ = doc;
	_tmp23_ = gtk_text_buffer_get_insert (_tmp22_);
	gtk_text_view_scroll_to_mark ((GtkTextView*) _tmp21_, _tmp23_, (gdouble) 0, TRUE, (gdouble) 1, (gdouble) 0);
	_tmp24_ = type;
	structure_set_actions_sensitivity (self, _tmp24_);
	if (!first_select) {
		result = TRUE;
		_g_object_unref0 (doc);
		_g_object_unref0 (mark);
		return result;
	}
	_tmp25_ = tree_iter;
	structure_select_simple_list_item (self, &_tmp25_);
	result = TRUE;
	_g_object_unref0 (doc);
	_g_object_unref0 (mark);
	return result;
}


static void structure_select_simple_list_item (Structure* self, GtkTreeIter* tree_iter) {
	gboolean _tmp0_;
	GtkTreeView* _tmp1_;
	GtkTreeSelection* _tmp2_ = NULL;
	GtkTreeSelection* _tmp3_;
	GtkTreeSelection* list_select;
	GtkTreeSelection* _tmp4_;
	StructType type = 0;
	StructureModel* _tmp5_;
	GtkTreeIter _tmp6_;
	StructType _tmp7_;
	StructType* _tmp8_;
	gint _tmp8__length1;
	StructureModel* _tmp9_;
	GtkTreeIter _tmp10_;
	gint _tmp11_ = 0;
	gint row_num;
	gint _tmp12_;
	gint _tmp13_;
	GtkTreePath* _tmp14_;
	GtkTreePath* list_path;
	GtkTreeSelection* _tmp15_;
	GtkTreePath* _tmp16_;
	GtkTreeView* _tmp17_;
	GtkTreePath* _tmp18_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (tree_iter != NULL);
	_tmp0_ = self->priv->_list_is_hidden;
	if (_tmp0_) {
		return;
	}
	_tmp1_ = self->priv->_list_view;
	_tmp2_ = gtk_tree_view_get_selection (_tmp1_);
	_tmp3_ = _g_object_ref0 (_tmp2_);
	list_select = _tmp3_;
	_tmp4_ = list_select;
	gtk_tree_selection_unselect_all (_tmp4_);
	_tmp5_ = self->priv->_model;
	_tmp6_ = *tree_iter;
	gtk_tree_model_get ((GtkTreeModel*) _tmp5_, &_tmp6_, STRUCT_COLUMN_TYPE, &type, -1, -1);
	_tmp7_ = type;
	_tmp8_ = self->priv->_current_list_types;
	_tmp8__length1 = self->priv->_current_list_types_length1;
	if (!_vala_struct_type_array_contains (_tmp8_, _tmp8__length1, _tmp7_)) {
		_g_object_unref0 (list_select);
		return;
	}
	_tmp9_ = self->priv->_model;
	_tmp10_ = *tree_iter;
	_tmp11_ = structure_model_get_list_num_from_tree_iter (_tmp9_, &_tmp10_);
	row_num = _tmp11_;
	_tmp12_ = row_num;
	if (_tmp12_ == (-1)) {
		_g_object_unref0 (list_select);
		return;
	}
	_tmp13_ = row_num;
	_tmp14_ = gtk_tree_path_new_from_indices (_tmp13_, -1, -1);
	list_path = _tmp14_;
	self->priv->_first_select = FALSE;
	_tmp15_ = list_select;
	_tmp16_ = list_path;
	gtk_tree_selection_select_path (_tmp15_, _tmp16_);
	_tmp17_ = self->priv->_list_view;
	_tmp18_ = list_path;
	gtk_tree_view_scroll_to_cell (_tmp17_, _tmp18_, NULL, FALSE, (gfloat) 0, (gfloat) 0);
	_gtk_tree_path_free0 (list_path);
	_g_object_unref0 (list_select);
}


static void structure_show_active_document (Structure* self) {
	MainWindow* _tmp0_;
	Document* _tmp1_;
	Document* _tmp2_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->_main_window;
	_tmp1_ = main_window_get_active_document (_tmp0_);
	_tmp2_ = _tmp1_;
	structure_show_document (self, _tmp2_, FALSE);
}


static void _structure_on_parsing_done_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
	structure_on_parsing_done (self);
}


static void structure_show_document (Structure* self, Document* doc, gboolean force_parse) {
	GtkTreeView* _tmp0_;
	DocumentStructure* _tmp1_;
	Document* _tmp5_;
	Document* _tmp6_;
	DocumentStructure* _tmp7_ = NULL;
	gboolean _tmp8_;
	DocumentStructure* _tmp10_;
	gboolean _tmp11_;
	gboolean _tmp12_;
	g_return_if_fail (self != NULL);
	structure_set_model (self, NULL);
	_tmp0_ = self->priv->_tree_view;
	gtk_tree_view_columns_autosize (_tmp0_);
	_tmp1_ = self->priv->_document_structure;
	if (_tmp1_ != NULL) {
		DocumentStructure* _tmp2_;
		guint _tmp3_ = 0U;
		GQuark _tmp4_ = 0U;
		_tmp2_ = self->priv->_document_structure;
		g_signal_parse_name ("notify::parsing-done", G_TYPE_OBJECT, &_tmp3_, &_tmp4_, TRUE);
		g_signal_handlers_disconnect_matched ((GObject*) _tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_DETAIL | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, _tmp4_, NULL, (GCallback) _structure_on_parsing_done_g_object_notify, self);
	}
	_tmp5_ = doc;
	if (_tmp5_ == NULL) {
		_g_object_unref0 (self->priv->_document_structure);
		self->priv->_document_structure = NULL;
		return;
	}
	_tmp6_ = doc;
	_tmp7_ = document_get_structure (_tmp6_);
	_g_object_unref0 (self->priv->_document_structure);
	self->priv->_document_structure = _tmp7_;
	_tmp8_ = force_parse;
	if (_tmp8_) {
		DocumentStructure* _tmp9_;
		_tmp9_ = self->priv->_document_structure;
		document_structure_parse (_tmp9_);
	}
	_tmp10_ = self->priv->_document_structure;
	_tmp11_ = document_structure_get_parsing_done (_tmp10_);
	_tmp12_ = _tmp11_;
	if (_tmp12_) {
		structure_on_parsing_done (self);
	} else {
		DocumentStructure* _tmp13_;
		_tmp13_ = self->priv->_document_structure;
		g_signal_connect_object ((GObject*) _tmp13_, "notify::parsing-done", (GCallback) _structure_on_parsing_done_g_object_notify, self, 0);
	}
}


static void structure_on_parsing_done (Structure* self) {
	DocumentStructure* _tmp0_;
	DocumentStructure* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->_document_structure;
	g_return_if_fail (_tmp0_ != NULL);
	_tmp1_ = self->priv->_document_structure;
	_tmp2_ = document_structure_get_parsing_done (_tmp1_);
	_tmp3_ = _tmp2_;
	if (_tmp3_) {
		DocumentStructure* _tmp4_;
		StructureModel* _tmp5_ = NULL;
		StructureModel* _tmp6_;
		_tmp4_ = self->priv->_document_structure;
		_tmp5_ = document_structure_get_model (_tmp4_);
		_tmp6_ = _tmp5_;
		structure_set_model (self, _tmp6_);
		_g_object_unref0 (_tmp6_);
	}
}


static void structure_set_model (Structure* self, StructureModel* model) {
	StructureModel* _tmp0_;
	StructureModel* _tmp2_;
	StructureModel* _tmp3_;
	GtkTreeView* _tmp4_;
	StructureModel* _tmp5_;
	gboolean _tmp6_ = FALSE;
	StructureModel* _tmp7_;
	gboolean _tmp10_;
	g_return_if_fail (self != NULL);
	_tmp0_ = model;
	if (_tmp0_ != NULL) {
		StructureModel* _tmp1_;
		_tmp1_ = model;
		structure_model_set_emit_signals (_tmp1_, TRUE);
	}
	_tmp2_ = model;
	_tmp3_ = _g_object_ref0 (_tmp2_);
	_g_object_unref0 (self->priv->_model);
	self->priv->_model = _tmp3_;
	_tmp4_ = self->priv->_tree_view;
	_tmp5_ = model;
	gtk_tree_view_set_model (_tmp4_, (GtkTreeModel*) _tmp5_);
	_tmp7_ = model;
	if (_tmp7_ != NULL) {
		StructureModel* _tmp8_;
		guint _tmp9_ = 0U;
		_tmp8_ = model;
		_tmp9_ = structure_model_get_nb_items (_tmp8_);
		_tmp6_ = _tmp9_ <= ((guint) 2000);
	} else {
		_tmp6_ = FALSE;
	}
	_tmp10_ = _tmp6_;
	if (_tmp10_) {
		GtkTreeView* _tmp11_;
		_tmp11_ = self->priv->_tree_view;
		gtk_tree_view_expand_all (_tmp11_);
	}
	structure_populate_simple_list (self);
}


static void _structure_show_active_document_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
	structure_show_active_document (self);
}


void structure_connect_parsing (Structure* self) {
	MainWindow* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->_main_window;
	g_signal_connect_object ((GObject*) _tmp0_, "notify::active-document", (GCallback) _structure_show_active_document_g_object_notify, self, 0);
	structure_show_active_document (self);
}


void structure_disconnect_parsing (Structure* self) {
	MainWindow* _tmp0_;
	guint _tmp1_ = 0U;
	GQuark _tmp2_ = 0U;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->_main_window;
	g_signal_parse_name ("notify::active-document", G_TYPE_OBJECT, &_tmp1_, &_tmp2_, TRUE);
	g_signal_handlers_disconnect_matched ((GObject*) _tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_DETAIL | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, _tmp2_, NULL, (GCallback) _structure_show_active_document_g_object_notify, self);
}


gboolean structure_is_section (StructType type) {
	gboolean result = FALSE;
	StructType _tmp0_;
	_tmp0_ = type;
	result = _tmp0_ <= STRUCT_TYPE_SUBPARAGRAPH;
	return result;
}


/*************************************************************************/
static void structure_show_popup_menu (Structure* self, GdkEventButton* event) {
	GdkEventButton* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = event;
	if (_tmp0_ != NULL) {
		GtkMenu* _tmp1_;
		GdkEventButton* _tmp2_;
		guint _tmp3_;
		GdkEventButton* _tmp4_;
		guint32 _tmp5_;
		_tmp1_ = self->priv->_popup_menu;
		_tmp2_ = event;
		_tmp3_ = (*_tmp2_).button;
		_tmp4_ = event;
		_tmp5_ = (*_tmp4_).time;
		gtk_menu_popup (_tmp1_, NULL, NULL, NULL, NULL, _tmp3_, _tmp5_);
	} else {
		GtkMenu* _tmp6_;
		guint32 _tmp7_ = 0U;
		_tmp6_ = self->priv->_popup_menu;
		_tmp7_ = gtk_get_current_event_time ();
		gtk_menu_popup (_tmp6_, NULL, NULL, NULL, NULL, (guint) 0, _tmp7_);
	}
}


static void structure_set_actions_sensitivity (Structure* self, StructType type) {
	GtkAction* _tmp0_;
	GtkAction* _tmp1_;
	GtkAction* _tmp2_;
	GtkAction* _tmp3_;
	GtkAction* _tmp4_;
	GtkAction* _tmp5_;
	gboolean _tmp6_ = FALSE;
	StructType _tmp7_;
	GtkAction* _tmp9_;
	gboolean _tmp10_;
	GtkAction* _tmp11_;
	StructType _tmp12_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->_action_all_menu;
	gtk_action_set_sensitive (_tmp0_, TRUE);
	_tmp1_ = self->priv->_action_cut;
	gtk_action_set_sensitive (_tmp1_, TRUE);
	_tmp2_ = self->priv->_action_copy;
	gtk_action_set_sensitive (_tmp2_, TRUE);
	_tmp3_ = self->priv->_action_delete;
	gtk_action_set_sensitive (_tmp3_, TRUE);
	_tmp4_ = self->priv->_action_select;
	gtk_action_set_sensitive (_tmp4_, TRUE);
	_tmp5_ = self->priv->_action_comment;
	gtk_action_set_sensitive (_tmp5_, TRUE);
	_tmp7_ = type;
	if (STRUCT_TYPE_PART < _tmp7_) {
		StructType _tmp8_;
		_tmp8_ = type;
		_tmp6_ = _tmp8_ <= STRUCT_TYPE_SUBPARAGRAPH;
	} else {
		_tmp6_ = FALSE;
	}
	_tmp9_ = self->priv->_action_shift_left;
	_tmp10_ = _tmp6_;
	gtk_action_set_sensitive (_tmp9_, _tmp10_);
	_tmp11_ = self->priv->_action_shift_right;
	_tmp12_ = type;
	gtk_action_set_sensitive (_tmp11_, _tmp12_ < STRUCT_TYPE_SUBPARAGRAPH);
}


void structure_do_action (Structure* self, StructAction action_type) {
	GtkTreeIter selected_iter = {0};
	GtkTreeView* _tmp0_;
	GtkTreeIter _tmp1_ = {0};
	gint _tmp2_ = 0;
	gint selected_row;
	gint _tmp3_;
	gboolean refresh_simple_list;
	gboolean _tmp24_;
	GError * _inner_error_ = NULL;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->_tree_view;
	_tmp2_ = utils_get_selected_row (_tmp0_, &_tmp1_);
	selected_iter = _tmp1_;
	selected_row = _tmp2_;
	_tmp3_ = selected_row;
	g_return_if_fail (_tmp3_ != (-1));
	refresh_simple_list = FALSE;
	{
		DocumentStructure* _tmp4_;
		StructAction _tmp5_;
		GtkTreeIter _tmp6_;
		gboolean _tmp7_ = FALSE;
		_tmp4_ = self->priv->_document_structure;
		_tmp5_ = action_type;
		_tmp6_ = selected_iter;
		document_structure_do_action (_tmp4_, _tmp5_, &_tmp6_, &_tmp7_, &_inner_error_);
		refresh_simple_list = _tmp7_;
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == STRUCT_ERROR) {
				goto __catch61_struct_error;
			}
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	goto __finally61;
	__catch61_struct_error:
	{
		GError* e = NULL;
		MainWindow* _tmp8_;
		const gchar* _tmp9_ = NULL;
		StructAction _tmp10_;
		gchar* _tmp11_ = NULL;
		gchar* _tmp12_;
		GtkMessageDialog* _tmp13_;
		GtkMessageDialog* _tmp14_;
		GtkMessageDialog* _tmp15_;
		GtkMessageDialog* dialog;
		GError* _tmp16_;
		GtkMessageDialog* _tmp22_;
		GtkMessageDialog* _tmp23_;
		e = _inner_error_;
		_inner_error_ = NULL;
		_tmp8_ = self->priv->_main_window;
		_tmp9_ = _ ("Structure action error: %s");
		_tmp10_ = action_type;
		_tmp11_ = structure_get_action_name (_tmp10_);
		_tmp12_ = _tmp11_;
		_tmp13_ = (GtkMessageDialog*) gtk_message_dialog_new ((GtkWindow*) _tmp8_, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _tmp9_, _tmp12_);
		_tmp14_ = g_object_ref_sink (_tmp13_);
		_tmp15_ = _tmp14_;
		_g_free0 (_tmp12_);
		dialog = _tmp15_;
		_tmp16_ = e;
		if (g_error_matches (_tmp16_, STRUCT_ERROR, STRUCT_ERROR_DATA_OUTDATED)) {
			GtkMessageDialog* _tmp17_;
			const gchar* _tmp18_ = NULL;
			_tmp17_ = dialog;
			_tmp18_ = _ ("The structure data seems outdated. Please refresh the structure.");
			g_object_set (_tmp17_, "secondary-text", _tmp18_, NULL);
		} else {
			GtkMessageDialog* _tmp19_;
			GError* _tmp20_;
			const gchar* _tmp21_;
			_tmp19_ = dialog;
			_tmp20_ = e;
			_tmp21_ = _tmp20_->message;
			g_object_set (_tmp19_, "secondary-text", _tmp21_, NULL);
		}
		_tmp22_ = dialog;
		gtk_dialog_run ((GtkDialog*) _tmp22_);
		_tmp23_ = dialog;
		gtk_object_destroy ((GtkObject*) _tmp23_);
		_g_object_unref0 (dialog);
		_g_error_free0 (e);
		return;
	}
	__finally61:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
		g_clear_error (&_inner_error_);
		return;
	}
	_tmp24_ = refresh_simple_list;
	if (_tmp24_) {
		structure_populate_simple_list (self);
	}
}


static gchar* structure_get_action_name (StructAction action_type) {
	gchar* result = NULL;
	gchar** _tmp0_;
	gint _tmp0__length1;
	gchar** _tmp30_;
	gint _tmp30__length1;
	StructAction _tmp31_;
	const gchar* _tmp32_;
	gchar* _tmp33_;
	_tmp0_ = structure__action_names;
	_tmp0__length1 = structure__action_names_length1;
	if (_tmp0_ == NULL) {
		gchar** _tmp1_ = NULL;
		gchar** _tmp2_;
		gint _tmp2__length1;
		const gchar* _tmp3_ = NULL;
		gchar* _tmp4_;
		gchar* _tmp5_;
		gchar** _tmp6_;
		gint _tmp6__length1;
		const gchar* _tmp7_ = NULL;
		gchar* _tmp8_;
		gchar* _tmp9_;
		gchar** _tmp10_;
		gint _tmp10__length1;
		const gchar* _tmp11_ = NULL;
		gchar* _tmp12_;
		gchar* _tmp13_;
		gchar** _tmp14_;
		gint _tmp14__length1;
		const gchar* _tmp15_ = NULL;
		gchar* _tmp16_;
		gchar* _tmp17_;
		gchar** _tmp18_;
		gint _tmp18__length1;
		const gchar* _tmp19_ = NULL;
		gchar* _tmp20_;
		gchar* _tmp21_;
		gchar** _tmp22_;
		gint _tmp22__length1;
		const gchar* _tmp23_ = NULL;
		gchar* _tmp24_;
		gchar* _tmp25_;
		gchar** _tmp26_;
		gint _tmp26__length1;
		const gchar* _tmp27_ = NULL;
		gchar* _tmp28_;
		gchar* _tmp29_;
		_tmp1_ = g_new0 (gchar*, STRUCT_ACTION_NB_ACTIONS + 1);
		structure__action_names = (_vala_array_free (structure__action_names, structure__action_names_length1, (GDestroyNotify) g_free), NULL);
		structure__action_names = _tmp1_;
		structure__action_names_length1 = STRUCT_ACTION_NB_ACTIONS;
		_structure__action_names_size_ = structure__action_names_length1;
		_tmp2_ = structure__action_names;
		_tmp2__length1 = structure__action_names_length1;
		_tmp3_ = _ ("cut");
		_tmp4_ = g_strdup (_tmp3_);
		_g_free0 (_tmp2_[STRUCT_ACTION_CUT]);
		_tmp2_[STRUCT_ACTION_CUT] = _tmp4_;
		_tmp5_ = _tmp2_[STRUCT_ACTION_CUT];
		_tmp6_ = structure__action_names;
		_tmp6__length1 = structure__action_names_length1;
		_tmp7_ = _ ("copy");
		_tmp8_ = g_strdup (_tmp7_);
		_g_free0 (_tmp6_[STRUCT_ACTION_COPY]);
		_tmp6_[STRUCT_ACTION_COPY] = _tmp8_;
		_tmp9_ = _tmp6_[STRUCT_ACTION_COPY];
		_tmp10_ = structure__action_names;
		_tmp10__length1 = structure__action_names_length1;
		_tmp11_ = _ ("delete");
		_tmp12_ = g_strdup (_tmp11_);
		_g_free0 (_tmp10_[STRUCT_ACTION_DELETE]);
		_tmp10_[STRUCT_ACTION_DELETE] = _tmp12_;
		_tmp13_ = _tmp10_[STRUCT_ACTION_DELETE];
		_tmp14_ = structure__action_names;
		_tmp14__length1 = structure__action_names_length1;
		_tmp15_ = _ ("select");
		_tmp16_ = g_strdup (_tmp15_);
		_g_free0 (_tmp14_[STRUCT_ACTION_SELECT]);
		_tmp14_[STRUCT_ACTION_SELECT] = _tmp16_;
		_tmp17_ = _tmp14_[STRUCT_ACTION_SELECT];
		_tmp18_ = structure__action_names;
		_tmp18__length1 = structure__action_names_length1;
		_tmp19_ = _ ("comment");
		_tmp20_ = g_strdup (_tmp19_);
		_g_free0 (_tmp18_[STRUCT_ACTION_COMMENT]);
		_tmp18_[STRUCT_ACTION_COMMENT] = _tmp20_;
		_tmp21_ = _tmp18_[STRUCT_ACTION_COMMENT];
		_tmp22_ = structure__action_names;
		_tmp22__length1 = structure__action_names_length1;
		_tmp23_ = _ ("shift left");
		_tmp24_ = g_strdup (_tmp23_);
		_g_free0 (_tmp22_[STRUCT_ACTION_SHIFT_LEFT]);
		_tmp22_[STRUCT_ACTION_SHIFT_LEFT] = _tmp24_;
		_tmp25_ = _tmp22_[STRUCT_ACTION_SHIFT_LEFT];
		_tmp26_ = structure__action_names;
		_tmp26__length1 = structure__action_names_length1;
		_tmp27_ = _ ("shift right");
		_tmp28_ = g_strdup (_tmp27_);
		_g_free0 (_tmp26_[STRUCT_ACTION_SHIFT_RIGHT]);
		_tmp26_[STRUCT_ACTION_SHIFT_RIGHT] = _tmp28_;
		_tmp29_ = _tmp26_[STRUCT_ACTION_SHIFT_RIGHT];
	}
	_tmp30_ = structure__action_names;
	_tmp30__length1 = structure__action_names_length1;
	_tmp31_ = action_type;
	_tmp32_ = _tmp30_[_tmp31_];
	_tmp33_ = g_strdup (_tmp32_);
	result = _tmp33_;
	return result;
}


gchar* structure_get_icon_from_type (StructType type) {
	gchar* result = NULL;
	gchar** _tmp0_;
	gint _tmp0__length1;
	gchar** _tmp44_;
	gint _tmp44__length1;
	StructType _tmp45_;
	const gchar* _tmp46_;
	gchar* _tmp47_;
	_tmp0_ = structure__icons;
	_tmp0__length1 = structure__icons_length1;
	if (_tmp0_ == NULL) {
		gchar** _tmp1_ = NULL;
		gchar** _tmp2_;
		gint _tmp2__length1;
		gchar* _tmp3_;
		gchar* _tmp4_;
		gchar** _tmp5_;
		gint _tmp5__length1;
		gchar* _tmp6_;
		gchar* _tmp7_;
		gchar** _tmp8_;
		gint _tmp8__length1;
		gchar* _tmp9_;
		gchar* _tmp10_;
		gchar** _tmp11_;
		gint _tmp11__length1;
		gchar* _tmp12_;
		gchar* _tmp13_;
		gchar** _tmp14_;
		gint _tmp14__length1;
		gchar* _tmp15_;
		gchar* _tmp16_;
		gchar** _tmp17_;
		gint _tmp17__length1;
		gchar* _tmp18_;
		gchar* _tmp19_;
		gchar** _tmp20_;
		gint _tmp20__length1;
		gchar* _tmp21_;
		gchar* _tmp22_;
		gchar** _tmp23_;
		gint _tmp23__length1;
		gchar* _tmp24_;
		gchar* _tmp25_;
		gchar** _tmp26_;
		gint _tmp26__length1;
		gchar* _tmp27_;
		gchar* _tmp28_;
		gchar** _tmp29_;
		gint _tmp29__length1;
		gchar* _tmp30_;
		gchar* _tmp31_;
		gchar** _tmp32_;
		gint _tmp32__length1;
		gchar* _tmp33_;
		gchar* _tmp34_;
		gchar** _tmp35_;
		gint _tmp35__length1;
		gchar* _tmp36_;
		gchar* _tmp37_;
		gchar** _tmp38_;
		gint _tmp38__length1;
		gchar* _tmp39_;
		gchar* _tmp40_;
		gchar** _tmp41_;
		gint _tmp41__length1;
		gchar* _tmp42_;
		gchar* _tmp43_;
		_tmp1_ = g_new0 (gchar*, STRUCT_TYPE_NB_TYPES + 1);
		structure__icons = (_vala_array_free (structure__icons, structure__icons_length1, (GDestroyNotify) g_free), NULL);
		structure__icons = _tmp1_;
		structure__icons_length1 = STRUCT_TYPE_NB_TYPES;
		_structure__icons_size_ = structure__icons_length1;
		_tmp2_ = structure__icons;
		_tmp2__length1 = structure__icons_length1;
		_tmp3_ = g_strdup ("tree_part");
		_g_free0 (_tmp2_[STRUCT_TYPE_PART]);
		_tmp2_[STRUCT_TYPE_PART] = _tmp3_;
		_tmp4_ = _tmp2_[STRUCT_TYPE_PART];
		_tmp5_ = structure__icons;
		_tmp5__length1 = structure__icons_length1;
		_tmp6_ = g_strdup ("tree_chapter");
		_g_free0 (_tmp5_[STRUCT_TYPE_CHAPTER]);
		_tmp5_[STRUCT_TYPE_CHAPTER] = _tmp6_;
		_tmp7_ = _tmp5_[STRUCT_TYPE_CHAPTER];
		_tmp8_ = structure__icons;
		_tmp8__length1 = structure__icons_length1;
		_tmp9_ = g_strdup ("tree_section");
		_g_free0 (_tmp8_[STRUCT_TYPE_SECTION]);
		_tmp8_[STRUCT_TYPE_SECTION] = _tmp9_;
		_tmp10_ = _tmp8_[STRUCT_TYPE_SECTION];
		_tmp11_ = structure__icons;
		_tmp11__length1 = structure__icons_length1;
		_tmp12_ = g_strdup ("tree_subsection");
		_g_free0 (_tmp11_[STRUCT_TYPE_SUBSECTION]);
		_tmp11_[STRUCT_TYPE_SUBSECTION] = _tmp12_;
		_tmp13_ = _tmp11_[STRUCT_TYPE_SUBSECTION];
		_tmp14_ = structure__icons;
		_tmp14__length1 = structure__icons_length1;
		_tmp15_ = g_strdup ("tree_subsubsection");
		_g_free0 (_tmp14_[STRUCT_TYPE_SUBSUBSECTION]);
		_tmp14_[STRUCT_TYPE_SUBSUBSECTION] = _tmp15_;
		_tmp16_ = _tmp14_[STRUCT_TYPE_SUBSUBSECTION];
		_tmp17_ = structure__icons;
		_tmp17__length1 = structure__icons_length1;
		_tmp18_ = g_strdup ("tree_paragraph");
		_g_free0 (_tmp17_[STRUCT_TYPE_PARAGRAPH]);
		_tmp17_[STRUCT_TYPE_PARAGRAPH] = _tmp18_;
		_tmp19_ = _tmp17_[STRUCT_TYPE_PARAGRAPH];
		_tmp20_ = structure__icons;
		_tmp20__length1 = structure__icons_length1;
		_tmp21_ = g_strdup ("tree_paragraph");
		_g_free0 (_tmp20_[STRUCT_TYPE_SUBPARAGRAPH]);
		_tmp20_[STRUCT_TYPE_SUBPARAGRAPH] = _tmp21_;
		_tmp22_ = _tmp20_[STRUCT_TYPE_SUBPARAGRAPH];
		_tmp23_ = structure__icons;
		_tmp23__length1 = structure__icons_length1;
		_tmp24_ = g_strdup ("tree_label");
		_g_free0 (_tmp23_[STRUCT_TYPE_LABEL]);
		_tmp23_[STRUCT_TYPE_LABEL] = _tmp24_;
		_tmp25_ = _tmp23_[STRUCT_TYPE_LABEL];
		_tmp26_ = structure__icons;
		_tmp26__length1 = structure__icons_length1;
		_tmp27_ = g_strdup ("tree_todo");
		_g_free0 (_tmp26_[STRUCT_TYPE_TODO]);
		_tmp26_[STRUCT_TYPE_TODO] = _tmp27_;
		_tmp28_ = _tmp26_[STRUCT_TYPE_TODO];
		_tmp29_ = structure__icons;
		_tmp29__length1 = structure__icons_length1;
		_tmp30_ = g_strdup ("tree_todo");
		_g_free0 (_tmp29_[STRUCT_TYPE_FIXME]);
		_tmp29_[STRUCT_TYPE_FIXME] = _tmp30_;
		_tmp31_ = _tmp29_[STRUCT_TYPE_FIXME];
		_tmp32_ = structure__icons;
		_tmp32__length1 = structure__icons_length1;
		_tmp33_ = g_strdup ("table");
		_g_free0 (_tmp32_[STRUCT_TYPE_TABLE]);
		_tmp32_[STRUCT_TYPE_TABLE] = _tmp33_;
		_tmp34_ = _tmp32_[STRUCT_TYPE_TABLE];
		_tmp35_ = structure__icons;
		_tmp35__length1 = structure__icons_length1;
		_tmp36_ = g_strdup ("image");
		_g_free0 (_tmp35_[STRUCT_TYPE_FIGURE]);
		_tmp35_[STRUCT_TYPE_FIGURE] = _tmp36_;
		_tmp37_ = _tmp35_[STRUCT_TYPE_FIGURE];
		_tmp38_ = structure__icons;
		_tmp38__length1 = structure__icons_length1;
		_tmp39_ = g_strdup ("image");
		_g_free0 (_tmp38_[STRUCT_TYPE_IMAGE]);
		_tmp38_[STRUCT_TYPE_IMAGE] = _tmp39_;
		_tmp40_ = _tmp38_[STRUCT_TYPE_IMAGE];
		_tmp41_ = structure__icons;
		_tmp41__length1 = structure__icons_length1;
		_tmp42_ = g_strdup (GTK_STOCK_FILE);
		_g_free0 (_tmp41_[STRUCT_TYPE_INCLUDE]);
		_tmp41_[STRUCT_TYPE_INCLUDE] = _tmp42_;
		_tmp43_ = _tmp41_[STRUCT_TYPE_INCLUDE];
	}
	_tmp44_ = structure__icons;
	_tmp44__length1 = structure__icons_length1;
	_tmp45_ = type;
	_tmp46_ = _tmp44_[_tmp45_];
	_tmp47_ = g_strdup (_tmp46_);
	result = _tmp47_;
	return result;
}


gchar* structure_get_type_name (StructType type) {
	gchar* result = NULL;
	gchar** _tmp0_;
	gint _tmp0__length1;
	gchar** _tmp56_;
	gint _tmp56__length1;
	StructType _tmp57_;
	const gchar* _tmp58_;
	gchar* _tmp59_;
	_tmp0_ = structure__names;
	_tmp0__length1 = structure__names_length1;
	if (_tmp0_ == NULL) {
		gchar** _tmp1_ = NULL;
		gchar** _tmp2_;
		gint _tmp2__length1;
		const gchar* _tmp3_ = NULL;
		gchar* _tmp4_;
		gchar* _tmp5_;
		gchar** _tmp6_;
		gint _tmp6__length1;
		const gchar* _tmp7_ = NULL;
		gchar* _tmp8_;
		gchar* _tmp9_;
		gchar** _tmp10_;
		gint _tmp10__length1;
		const gchar* _tmp11_ = NULL;
		gchar* _tmp12_;
		gchar* _tmp13_;
		gchar** _tmp14_;
		gint _tmp14__length1;
		const gchar* _tmp15_ = NULL;
		gchar* _tmp16_;
		gchar* _tmp17_;
		gchar** _tmp18_;
		gint _tmp18__length1;
		const gchar* _tmp19_ = NULL;
		gchar* _tmp20_;
		gchar* _tmp21_;
		gchar** _tmp22_;
		gint _tmp22__length1;
		const gchar* _tmp23_ = NULL;
		gchar* _tmp24_;
		gchar* _tmp25_;
		gchar** _tmp26_;
		gint _tmp26__length1;
		const gchar* _tmp27_ = NULL;
		gchar* _tmp28_;
		gchar* _tmp29_;
		gchar** _tmp30_;
		gint _tmp30__length1;
		const gchar* _tmp31_ = NULL;
		gchar* _tmp32_;
		gchar* _tmp33_;
		gchar** _tmp34_;
		gint _tmp34__length1;
		gchar* _tmp35_;
		gchar* _tmp36_;
		gchar** _tmp37_;
		gint _tmp37__length1;
		gchar* _tmp38_;
		gchar* _tmp39_;
		gchar** _tmp40_;
		gint _tmp40__length1;
		const gchar* _tmp41_ = NULL;
		gchar* _tmp42_;
		gchar* _tmp43_;
		gchar** _tmp44_;
		gint _tmp44__length1;
		const gchar* _tmp45_ = NULL;
		gchar* _tmp46_;
		gchar* _tmp47_;
		gchar** _tmp48_;
		gint _tmp48__length1;
		const gchar* _tmp49_ = NULL;
		gchar* _tmp50_;
		gchar* _tmp51_;
		gchar** _tmp52_;
		gint _tmp52__length1;
		const gchar* _tmp53_ = NULL;
		gchar* _tmp54_;
		gchar* _tmp55_;
		_tmp1_ = g_new0 (gchar*, STRUCT_TYPE_NB_TYPES + 1);
		structure__names = (_vala_array_free (structure__names, structure__names_length1, (GDestroyNotify) g_free), NULL);
		structure__names = _tmp1_;
		structure__names_length1 = STRUCT_TYPE_NB_TYPES;
		_structure__names_size_ = structure__names_length1;
		_tmp2_ = structure__names;
		_tmp2__length1 = structure__names_length1;
		_tmp3_ = _ ("Part");
		_tmp4_ = g_strdup (_tmp3_);
		_g_free0 (_tmp2_[STRUCT_TYPE_PART]);
		_tmp2_[STRUCT_TYPE_PART] = _tmp4_;
		_tmp5_ = _tmp2_[STRUCT_TYPE_PART];
		_tmp6_ = structure__names;
		_tmp6__length1 = structure__names_length1;
		_tmp7_ = _ ("Chapter");
		_tmp8_ = g_strdup (_tmp7_);
		_g_free0 (_tmp6_[STRUCT_TYPE_CHAPTER]);
		_tmp6_[STRUCT_TYPE_CHAPTER] = _tmp8_;
		_tmp9_ = _tmp6_[STRUCT_TYPE_CHAPTER];
		_tmp10_ = structure__names;
		_tmp10__length1 = structure__names_length1;
		_tmp11_ = _ ("Section");
		_tmp12_ = g_strdup (_tmp11_);
		_g_free0 (_tmp10_[STRUCT_TYPE_SECTION]);
		_tmp10_[STRUCT_TYPE_SECTION] = _tmp12_;
		_tmp13_ = _tmp10_[STRUCT_TYPE_SECTION];
		_tmp14_ = structure__names;
		_tmp14__length1 = structure__names_length1;
		_tmp15_ = _ ("Sub-section");
		_tmp16_ = g_strdup (_tmp15_);
		_g_free0 (_tmp14_[STRUCT_TYPE_SUBSECTION]);
		_tmp14_[STRUCT_TYPE_SUBSECTION] = _tmp16_;
		_tmp17_ = _tmp14_[STRUCT_TYPE_SUBSECTION];
		_tmp18_ = structure__names;
		_tmp18__length1 = structure__names_length1;
		_tmp19_ = _ ("Sub-sub-section");
		_tmp20_ = g_strdup (_tmp19_);
		_g_free0 (_tmp18_[STRUCT_TYPE_SUBSUBSECTION]);
		_tmp18_[STRUCT_TYPE_SUBSUBSECTION] = _tmp20_;
		_tmp21_ = _tmp18_[STRUCT_TYPE_SUBSUBSECTION];
		_tmp22_ = structure__names;
		_tmp22__length1 = structure__names_length1;
		_tmp23_ = _ ("Paragraph");
		_tmp24_ = g_strdup (_tmp23_);
		_g_free0 (_tmp22_[STRUCT_TYPE_PARAGRAPH]);
		_tmp22_[STRUCT_TYPE_PARAGRAPH] = _tmp24_;
		_tmp25_ = _tmp22_[STRUCT_TYPE_PARAGRAPH];
		_tmp26_ = structure__names;
		_tmp26__length1 = structure__names_length1;
		_tmp27_ = _ ("Sub-paragraph");
		_tmp28_ = g_strdup (_tmp27_);
		_g_free0 (_tmp26_[STRUCT_TYPE_SUBPARAGRAPH]);
		_tmp26_[STRUCT_TYPE_SUBPARAGRAPH] = _tmp28_;
		_tmp29_ = _tmp26_[STRUCT_TYPE_SUBPARAGRAPH];
		_tmp30_ = structure__names;
		_tmp30__length1 = structure__names_length1;
		_tmp31_ = _ ("Label");
		_tmp32_ = g_strdup (_tmp31_);
		_g_free0 (_tmp30_[STRUCT_TYPE_LABEL]);
		_tmp30_[STRUCT_TYPE_LABEL] = _tmp32_;
		_tmp33_ = _tmp30_[STRUCT_TYPE_LABEL];
		_tmp34_ = structure__names;
		_tmp34__length1 = structure__names_length1;
		_tmp35_ = g_strdup ("TODO");
		_g_free0 (_tmp34_[STRUCT_TYPE_TODO]);
		_tmp34_[STRUCT_TYPE_TODO] = _tmp35_;
		_tmp36_ = _tmp34_[STRUCT_TYPE_TODO];
		_tmp37_ = structure__names;
		_tmp37__length1 = structure__names_length1;
		_tmp38_ = g_strdup ("FIXME");
		_g_free0 (_tmp37_[STRUCT_TYPE_FIXME]);
		_tmp37_[STRUCT_TYPE_FIXME] = _tmp38_;
		_tmp39_ = _tmp37_[STRUCT_TYPE_FIXME];
		_tmp40_ = structure__names;
		_tmp40__length1 = structure__names_length1;
		_tmp41_ = _ ("Table");
		_tmp42_ = g_strdup (_tmp41_);
		_g_free0 (_tmp40_[STRUCT_TYPE_TABLE]);
		_tmp40_[STRUCT_TYPE_TABLE] = _tmp42_;
		_tmp43_ = _tmp40_[STRUCT_TYPE_TABLE];
		_tmp44_ = structure__names;
		_tmp44__length1 = structure__names_length1;
		_tmp45_ = _ ("Figure");
		_tmp46_ = g_strdup (_tmp45_);
		_g_free0 (_tmp44_[STRUCT_TYPE_FIGURE]);
		_tmp44_[STRUCT_TYPE_FIGURE] = _tmp46_;
		_tmp47_ = _tmp44_[STRUCT_TYPE_FIGURE];
		_tmp48_ = structure__names;
		_tmp48__length1 = structure__names_length1;
		_tmp49_ = _ ("Image");
		_tmp50_ = g_strdup (_tmp49_);
		_g_free0 (_tmp48_[STRUCT_TYPE_IMAGE]);
		_tmp48_[STRUCT_TYPE_IMAGE] = _tmp50_;
		_tmp51_ = _tmp48_[STRUCT_TYPE_IMAGE];
		_tmp52_ = structure__names;
		_tmp52__length1 = structure__names_length1;
		_tmp53_ = _ ("File included");
		_tmp54_ = g_strdup (_tmp53_);
		_g_free0 (_tmp52_[STRUCT_TYPE_INCLUDE]);
		_tmp52_[STRUCT_TYPE_INCLUDE] = _tmp54_;
		_tmp55_ = _tmp52_[STRUCT_TYPE_INCLUDE];
	}
	_tmp56_ = structure__names;
	_tmp56__length1 = structure__names_length1;
	_tmp57_ = type;
	_tmp58_ = _tmp56_[_tmp57_];
	_tmp59_ = g_strdup (_tmp58_);
	result = _tmp59_;
	return result;
}


static void structure_class_init (StructureClass * klass) {
	structure_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (StructurePrivate));
	G_OBJECT_CLASS (klass)->finalize = structure_finalize;
}


static void structure_instance_init (Structure * self) {
	GtkToggleButton** _tmp0_ = NULL;
	self->priv = STRUCTURE_GET_PRIVATE (self);
	_tmp0_ = g_new0 (GtkToggleButton*, 0 + 1);
	self->priv->_simple_list_buttons = _tmp0_;
	self->priv->_simple_list_buttons_length1 = 0;
	self->priv->__simple_list_buttons_size_ = self->priv->_simple_list_buttons_length1;
	self->priv->_document_structure = NULL;
	self->priv->_model = NULL;
	self->priv->_list_is_hidden = TRUE;
	self->priv->_first_select = TRUE;
}


static void structure_finalize (GObject* obj) {
	Structure * self;
	self = STRUCTURE (obj);
	_g_object_unref0 (self->priv->_popup_menu);
	_g_object_unref0 (self->priv->_action_all_menu);
	_g_object_unref0 (self->priv->_action_cut);
	_g_object_unref0 (self->priv->_action_copy);
	_g_object_unref0 (self->priv->_action_delete);
	_g_object_unref0 (self->priv->_action_select);
	_g_object_unref0 (self->priv->_action_comment);
	_g_object_unref0 (self->priv->_action_shift_left);
	_g_object_unref0 (self->priv->_action_shift_right);
	self->priv->_simple_list_buttons = (_vala_array_free (self->priv->_simple_list_buttons, self->priv->_simple_list_buttons_length1, (GDestroyNotify) g_object_unref), NULL);
	_g_object_unref0 (self->priv->_vpaned);
	_g_object_unref0 (self->priv->_tree_view);
	_g_object_unref0 (self->priv->_document_structure);
	_g_object_unref0 (self->priv->_model);
	_g_object_unref0 (self->priv->_list_view);
	_g_object_unref0 (self->priv->_list_view_sw);
	_g_object_unref0 (self->priv->_list_store);
	self->priv->_current_list_types = (g_free (self->priv->_current_list_types), NULL);
	G_OBJECT_CLASS (structure_parent_class)->finalize (obj);
}


GType structure_get_type (void) {
	static volatile gsize structure_type_id__volatile = 0;
	if (g_once_init_enter (&structure_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (StructureClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) structure_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Structure), 0, (GInstanceInitFunc) structure_instance_init, NULL };
		GType structure_type_id;
		structure_type_id = g_type_register_static (GTK_TYPE_VBOX, "Structure", &g_define_type_info, 0);
		g_once_init_leave (&structure_type_id__volatile, structure_type_id);
	}
	return structure_type_id__volatile;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}



